# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
from pyCGM2.Math import numeric
import logging
from pyCGM2.Utils import utils


#---------TESTS----------

def test_offset(value,acq,viconLabel, decimal=3):
    np.testing.assert_almost_equal(value,
    np.rad2deg(acq.GetMetaData().FindChild(utils.str("PROCESSING")).value().FindChild(utils.str(viconLabel)).value().GetInfo().ToDouble()[0]) , decimal = decimal)


def test_point(acq,RefLabel,LabelToTest,decimal = 3,init=-1,end=-1):
    init = 0  if init == -1 else  init - acq.GetFirstFrame()
    end = acq.GetLastFrame()- acq.GetFirstFrame()  if end == -1 else  end - acq.GetFirstFrame()

    np.testing.assert_almost_equal(acq.GetPoint(utils.str(RefLabel)).GetValues()[init:end,:],acq.GetPoint(utils.str(LabelToTest)).GetValues()[init:end,:],decimal = decimal)


def test_point_rms(acq,RefLabel,LabelToTest,threshold,init=-1,end=-1):
    init = 0  if init == -1 else  init - acq.GetFirstFrame()
    end = acq.GetLastFrame()- acq.GetFirstFrame()  if end == -1 else  end - acq.GetFirstFrame()

    np.testing.assert_array_less(numeric.rms( acq.GetPoint(utils.str(RefLabel)).GetValues()[init:end,:] -
        acq.GetPoint(utils.str(LabelToTest)).GetValues()[init:end,:], axis = 0),threshold)


#---------DISPLAY----------
def print_offset(value,acq,viconLabel, decimal=3):
    logging.info(" offset [%s] => %f ( my value) = %f ( reference)"%(viconLabel,
                            value,
                            np.rad2deg(acq.GetMetaData().FindChild(utils.str("PROCESSING")).value().FindChild(utils.str(viconLabel)).value().GetInfo().ToDouble()[0])))


def plotComparisonOfPoint(acq,label,suffix,title=None,init=-1,end=-1):
    init = 0  if init == -1 else  init - acq.GetFirstFrame()
    end = acq.GetLastFrame()- acq.GetFirstFrame()  if end == -1 else  end - acq.GetFirstFrame()

    fig = plt.figure(figsize=(10,4), dpi=100,facecolor="white")
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)
    ax1 = plt.subplot(1,3,1)
    ax2 = plt.subplot(1,3,2)
    ax3 = plt.subplot(1,3,3)

    ax1.plot(acq.GetPoint(utils.str(label)).GetValues()[init:end,0],"-r")
    ax1.plot(acq.GetPoint(utils.str(label+"_"+suffix)).GetValues()[init:end,0],"-b")


    ax2.plot(acq.GetPoint(utils.str(label)).GetValues()[init:end,1],"-r")
    ax2.plot(acq.GetPoint(utils.str(label+"_"+suffix)).GetValues()[init:end,1],"-b")

    ax3.plot(acq.GetPoint(utils.str(label)).GetValues()[init:end,2],"-r")
    ax3.plot(acq.GetPoint(utils.str(label+"_"+suffix)).GetValues()[init:end,2],"-b")

    if title is not None: plt.title(title)

    plt.show()

def plotValuesComparison(values0,values1):

    fig = plt.figure(figsize=(10,4), dpi=100,facecolor="white")
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)
    ax1 = plt.subplot(1,3,1)
    ax2 = plt.subplot(1,3,2)
    ax3 = plt.subplot(1,3,3)

    ax1.plot(values0[:,0],"-r")
    ax1.plot(values1[:,0],"-b")


    ax2.plot(values0[:,1],"-r")
    ax2.plot(values1[:,1],"-b")

    ax3.plot(values0[:,2],"-r")
    ax3.plot(values1[:,2],"-b")

    plt.show()

def plotComparison_MomentPanel(acq,suffix1,suffix2,side,title=None,init=-1,end=-1):


    if side=="Left":
        sideLetter = "L"
    elif side=="Right":
        sideLetter = "R"
    else:
        raise Exception ("side is Left or Right only")

    suffix1  = "_"+suffix1  if suffix1 is not None else ""
    suffix2  = "_"+suffix2  if suffix2 is not None else ""

    init = 0  if init == -1 else  init - acq.GetFirstFrame()
    end = end = acq.GetLastFrame()- acq.GetFirstFrame()  if end == -1 else  end - acq.GetFirstFrame()

    fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
    if title is not None: fig.suptitle(title)
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

    ax0 = plt.subplot(3,4,1)# Hip X extensor
    ax1 = plt.subplot(3,4,2)# Hip Y abductor
    ax2 = plt.subplot(3,4,3)# Hip Z rotation
    ax3 = plt.subplot(3,4,4)# Knee Z power

    ax4 = plt.subplot(3,4,5)# knee X extensor
    ax5 = plt.subplot(3,4,6)# knee Y abductor
    ax6 = plt.subplot(3,4,7)# knee Z rotation
    ax7 = plt.subplot(3,4,8)# knee Z power

    ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
    ax9 = plt.subplot(3,4,10)# ankle Y rotation
    ax10 = plt.subplot(3,4,11)# ankle Z everter
    ax11 = plt.subplot(3,4,12)# ankle Z power

    ax0.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix1)).GetValues()[init:end,0],"-r")
    ax0.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix2)).GetValues()[init:end,0],"-b")

    ax1.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix1)).GetValues()[init:end,1],"-r")
    ax1.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix2)).GetValues()[init:end,1],"-b")

    ax2.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix1)).GetValues()[init:end,2],"-r")
    ax2.plot(acq.GetPoint(utils.str(sideLetter+ "HipMoment"+suffix2)).GetValues()[init:end,2],"-b")


    ax4.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix1)).GetValues()[init:end,0],"-r")
    ax4.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix2)).GetValues()[init:end,0],"-b")

    ax5.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix1)).GetValues()[init:end,1],"-r")
    ax5.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix2)).GetValues()[init:end,1],"-b")

    ax6.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix1)).GetValues()[init:end,2],"-r")
    ax6.plot(acq.GetPoint(utils.str(sideLetter+ "KneeMoment"+suffix2)).GetValues()[init:end,2],"-b")


    ax8.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix1)).GetValues()[init:end,0],"-r")
    ax8.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix2)).GetValues()[init:end,0],"-b")

    ax9.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix1)).GetValues()[init:end,1],"-r")
    ax9.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix2)).GetValues()[init:end,1],"-b")

    ax10.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix1)).GetValues()[init:end,2],"-r")
    ax10.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleMoment"+suffix2)).GetValues()[init:end,2],"-b")

    plt.show()


def plotComparison_ForcePanel(acq,suffix1,suffix2,side,title=None,init=-1,end=-1):

    if side=="Left":
        sideLetter = "L"
    elif side=="Right":
        sideLetter = "R"
    else:
        raise Exception ("side is Left or Right only")


    suffix1  = "_"+suffix1  if suffix1 is not None else ""
    suffix2  = "_"+suffix2  if suffix2 is not None else ""

    init = 0  if init == -1 else  init - acq.GetFirstFrame()
    end = acq.GetLastFrame()- acq.GetFirstFrame()  if end == -1 else  end - acq.GetFirstFrame()


    fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
    if title is not None: fig.suptitle(title)
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

    ax0 = plt.subplot(3,4,1)# Hip X extensor
    ax1 = plt.subplot(3,4,2)# Hip Y abductor
    ax2 = plt.subplot(3,4,3)# Hip Z rotation
    ax3 = plt.subplot(3,4,4)# Knee Z power

    ax4 = plt.subplot(3,4,5)# knee X extensor
    ax5 = plt.subplot(3,4,6)# knee Y abductor
    ax6 = plt.subplot(3,4,7)# knee Z rotation
    ax7 = plt.subplot(3,4,8)# knee Z power

    ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
    ax9 = plt.subplot(3,4,10)# ankle Y rotation
    ax10 = plt.subplot(3,4,11)# ankle Z everter
    ax11 = plt.subplot(3,4,12)# ankle Z power

    ax0.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix1)).GetValues()[init:end,0],"-r")
    ax0.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix2)).GetValues()[init:end,0],"-b")

    ax1.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix1)).GetValues()[init:end,1],"-r")
    ax1.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix2)).GetValues()[init:end,1],"-b")

    ax2.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix1)).GetValues()[init:end,2],"-r")
    ax2.plot(acq.GetPoint(utils.str(sideLetter+ "HipForce"+suffix2)).GetValues()[init:end,2],"-b")


    ax4.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix1)).GetValues()[init:end,0],"-r")
    ax4.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix2)).GetValues()[init:end,0],"-b")

    ax5.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix1)).GetValues()[init:end,1],"-r")
    ax5.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix2)).GetValues()[init:end,1],"-b")

    ax6.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix1)).GetValues()[init:end,2],"-r")
    ax6.plot(acq.GetPoint(utils.str(sideLetter+ "KneeForce"+suffix2)).GetValues()[init:end,2],"-b")


    ax8.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix1)).GetValues()[init:end,0],"-r")
    ax8.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix2)).GetValues()[init:end,0],"-b")

    ax9.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix1)).GetValues()[init:end,1],"-r")
    ax9.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix2)).GetValues()[init:end,1],"-b")

    ax10.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix1)).GetValues()[init:end,2],"-r")
    ax10.plot(acq.GetPoint(utils.str(sideLetter+ "AnkleForce"+suffix2)).GetValues()[init:end,2],"-b")

    plt.show()



# -*- coding: utf-8 -*-
import numpy as np
import copy
import logging

from pyCGM2 import btk

import frame

from pyCGM2 import enums
from pyCGM2.Model import motion
from  pyCGM2.Tools import  btkTools
from pyCGM2.Math import  derivation
from  pyCGM2.Signal import signal_processing


class ClinicalDescriptor(object):
    def __init__(self,dataType,jointOrSegmentLabel, indexes,coefficients, offsets,**options):
        self.type = dataType
        self.label = jointOrSegmentLabel
        self.infos = dict()
        self.infos["SaggitalIndex"] = indexes[0]
        self.infos["CoronalIndex"] = indexes[1]
        self.infos["TransversalIndex"] = indexes[2]
        self.infos["SaggitalCoeff"] = coefficients[0]
        self.infos["CoronalCoeff"] = coefficients[1]
        self.infos["TransversalCoeff"] = coefficients[2]
        self.infos["SaggitalOffset"] = offsets[0]
        self.infos["CoronalOffset"] = offsets[1]
        self.infos["TransversalOffset"] = offsets[2]

        if "projection" in options:
            self.projection = options["projection"]


# -------- ABSTRACT MODEL ---------

class Model(object):
    """
        Abstract class `Model`.
        `Model` collects *segments*, *joints*, *body segment parameters* and present convenient model-componant accessors


    """

    def __init__(self):

        self.m_segmentCollection=[]
        self.m_jointCollection=[]
        self.mp=dict()
        self.mp_computed=dict()
        self.m_chains=dict()
        self.m_staticFilename=None

        self.m_properties=dict()
        self.m_properties["CalibrationParameters"]=dict()
        self.m_clinicalDescriptors= []
        self.m_csDefinitions = []
        self.m_bodypart=None
        self.m_centreOfMass=None

    def __repr__(self):
        return "Basis Model"

    def setBodyPart(self, bodypart):
        self.m_bodypart = bodypart

    def getBodyPart(self):
        return self.m_bodypart


    def setCentreOfMass(self,com):
        self.m_centreOfMass = com
    def getCentreOfMass(self):
        return self.m_centreOfMass


    def setProperty(self, propertyLabel,  value):
        self.m_properties[propertyLabel] = value

    def getProperty(self, propertyLabel):
        try:
            return self.m_properties[propertyLabel]
        except:
            raise ("property Label doesn t find")

    def setCalibrationProperty(self, propertyLabel,  value):
        self.m_properties["CalibrationParameters"][propertyLabel] = value

    def isProperty(self,label):
        return True if label in self.m_properties.keys() else False

    def isCalibrationProperty(self,label):
        return True if label in self.m_properties["CalibrationParameters"].keys() else False

    def checkCalibrationProperty(self,CalibrationParameterLabel,value):
        if self.isCalibrationProperty(CalibrationParameterLabel):
            if self.m_properties["CalibrationParameters"][CalibrationParameterLabel] == value:
                return True
            else:
                return False
        else:
            return False
            logging.warning("[pyCGM2] : CalibrationParameterLabel doesn t exist")

    def setStaticFilename(self,name):
        self.m_staticFilename=name

    def addChain(self,label,indexSegmentList):
        """
            Add a segment chain

            :Parameters:
                - `label` (str) - label of the chain
                - `indexSegmentList` (list of int) - index of segment

        """
        self.m_chains[label] = indexSegmentList

    def addJoint(self,label,proxLabel, distLabel, sequence, nodeLabel):
        """
            Add a joint

            :Parameters:
                - `label` (str) - label of the chain
                - `proxLabel` (str) - label of the proximal segment
                - `distLabel` (str) - label of the distal segment
                - `sequence` (str) - sequence angle

        """

        j=Joint( label, proxLabel,distLabel,sequence,nodeLabel)
        self.m_jointCollection.append(j)


    def addSegment(self,label,index,sideEnum, calibration_markers=[], tracking_markers=[],cloneOf=False):
        """
            Add a segment

            :Parameters:
                - `label` (str) - label of the segment
                - `index` (str) - index of the segment
                - `sideEnum` (pyCGM2.enums) - body side
                - `lst_markerlabel` (list of str) - sequence angle
                - `tracking_markers` (list of str) - sequence angle

        """

        seg=Segment(label,index,sideEnum,calibration_markers,tracking_markers)
        if cloneOf is True:
            seg.m_isCloneOf = True
        self.m_segmentCollection.append(seg)


    def updateSegmentFromCopy(self,targetLabel, segmentToCopy):
        """
            Update a segment from a copy of  another segment instance

            :Parameters:
                - `targetLabel` (str) - label of the segment
                - `segmentToCopy` (pyCGM2.Model.CGM2.model.Segment) - pyCGM2.Model.CGM2.model.Segment instance

        """
        copiedSegment = copy.deepcopy(segmentToCopy)
        copiedSegment.name = targetLabel

        isClone = True if self.getSegment(targetLabel).m_isCloneOf else False
        for i in range(0, len(self.m_segmentCollection)):
            if self.m_segmentCollection[i].name == targetLabel:
                self.m_segmentCollection[i] = copiedSegment
                self.m_segmentCollection[i].m_isCloneOf = isClone


    def removeSegment(self,segmentlabels):
        """
            remove `Segment` from its label

            :Parameters:
                - `label` (list) - segment labels to remove

            :Return:
                - `na` (pyCGM2.Model.CGM2.model.Segment) - pyCGM2.Model.CGM2.model.Segment instance


        """
        segment_list = [it for it in self.m_segmentCollection if it.name not in segmentlabels]
        self.m_segmentCollection = segment_list

    def removeJoint(self,jointlabels):
        """
            remove `joint` from its label

            :Parameters:
                - `jointlabels` (list) - joint labels to remove

            :Return:
                - `na` (pyCGM2.Model.CGM2.model.Segment) - pyCGM2.Model.CGM2.model.Segment instance


        """
        joint_list = [it for it in self.m_jointCollection if it.m_label not in jointlabels]
        self.m_jointCollection = joint_list



    def getSegment(self,label):
        """
            Get `Segment` from its label

            :Parameters:
                - `label` (str) - label of the Segment

            :Return:
                - `na` (pyCGM2.Model.CGM2.model.Segment) - pyCGM2.Model.CGM2.model.Segment instance


        """

        for it in self.m_segmentCollection:
            if it.name == label:
                return it

    def getSegmentIndex(self,label):
        """
            Get Segment index

            :Parameters:
                - `label` (str) - label of the Segment

            :Return:
                - `na` (int) - index


        """
        index=0
        for it in self.m_segmentCollection:
            if it.name == label:
                return index
            index+=1


    def getSegmentByIndex(self,index):
        """
            Get `Segment` from its index

            :Parameters:
                - `index` (int) - index of the Segment

            :Return:
                - `na` (pyCGM2.Model.CGM2.model.Segment) - pyCGM2.Model.CGM2.model.Segment instance


        """

        for it in self.m_segmentCollection:
            if it.index == index:
                return it

    def getSegmentList(self):
        """
            Get list of `Segment` labels

            :Parameters:


            :Return:
                - `labels` (list) - list of segment label


        """
        return [it.name for it in self.m_segmentCollection]


    def getJointList(self):
        """
            Get list of `Joint` labels

            :Parameters:


            :Return:
                - `labels` (list) - list of joint label


        """
        return [it.m_label for it in self.m_jointCollection]



    def getJoint(self,label):
        """
            Get `Joint` from its label

            :Parameters:
                - `label` (str) - label of the joint

            :Return:
                - `na` (pyCGM2.Model.CGM2.model.Joint) - pyCGM2.Model.CGM2.model.Joint instance


        """

        for it in self.m_jointCollection:
            if it.m_label == label:
                return it

    def addAnthropoInputParameters(self,iDict,optional=None):
        """
            Add measured anthropometric data to the model

            :Parameters:
               - `iDict` (dict) - requried anthropometric data
               - `optionalMp` (dict) - optional anthropometric data

        """

        self.mp=iDict

        if optional is not None:
            self.mp.update(optional)






    def decomposeTrackingMarkers(self,acq,TechnicalFrameLabel):

    #        decompose marker and keep a noisy signal along one axis only.
    #
    #        :Parameters:
    #         - `acq` (btk-acq) -
    #         - `TechnicalFrameLabel` (str) - label of the technical Frame
    #
    #        .. todo::
    #
    #         - comment travailler avec des referentiels techniques differents par segment
    #         - rajouter des exceptions


        for seg in self.m_segmentCollection:

            for marker in seg.m_tracking_markers:

                nodeTraj= seg.getReferential(TechnicalFrameLabel).getNodeTrajectory(marker)
                markersTraj =acq.GetPoint(marker).GetValues()

                markerTrajectoryX=np.array( [ markersTraj[:,0], nodeTraj[:,1], nodeTraj[:,2]]).T
                markerTrajectoryY=np.array( [ nodeTraj[:,0], markersTraj[:,1], nodeTraj[:,2]]).T
                markerTrajectoryZ=np.array( [ nodeTraj[:,0], nodeTraj[:,1], markersTraj[:,2]]).T


                btkTools.smartAppendPoint(acq,marker+"-X",markerTrajectoryX,PointType=btk.btkPoint.Marker, desc="")
                btkTools.smartAppendPoint(acq,marker+"-Y",markerTrajectoryY,PointType=btk.btkPoint.Marker, desc="")
                btkTools.smartAppendPoint(acq,marker+"-Z",markerTrajectoryZ,PointType=btk.btkPoint.Marker, desc="")


    def displayStaticCoordinateSystem(self,aquiStatic,  segmentLabel, targetPointLabel, referential = "Anatomic" ):
        """
            Display a coordinate system. Its Axis are represented by 3 virtual markers suffixed by (_X,_Y,_Z)

            :Parameters:
                - `aquiStatic` (btkAcquisition) - btkAcquisition instance from a static c3d
                - `segmentLabel` (str) - segment label
                - `targetPointLabel` (str) - label of the point defining axis limits
                - `referential` (str) - type of segment coordinate system you want dislay ( if other than *Anatomic*, Technical Coordinate system will be displayed )

        """

        seg=self.getSegment(segmentLabel)
        if referential == "Anatomic":
            ref =seg.anatomicalFrame
        else:
            ref = seg.getReferential("TF")

        val =  np.dot(ref.static.getRotation() , np.array([100.0,0.0,0.0])) + ref.static.getTranslation()
        btkTools.smartAppendPoint(aquiStatic,targetPointLabel+"_X",val*np.ones((aquiStatic.GetPointFrameNumber(),3)),desc="")
        val =  np.dot(ref.static.getRotation() , np.array([0.0,100.0,0.0])) + ref.static.getTranslation()
        btkTools.smartAppendPoint(aquiStatic,targetPointLabel+"_Y",val*np.ones((aquiStatic.GetPointFrameNumber(),3)),desc="")
        val =  np.dot(ref.static.getRotation() , np.array([0.0,0,100.0])) + ref.static.getTranslation()
        btkTools.smartAppendPoint(aquiStatic,targetPointLabel+"_Z",val*np.ones((aquiStatic.GetPointFrameNumber(),3)),desc="")

    def displayMotionCoordinateSystem(self,acqui,  segmentLabel, targetPointLabel, referential = "Anatomic" ):
        seg=self.getSegment(segmentLabel)
        valX=np.zeros((acqui.GetPointFrameNumber(),3))
        valY=np.zeros((acqui.GetPointFrameNumber(),3))
        valZ=np.zeros((acqui.GetPointFrameNumber(),3))


        if referential == "Anatomic":
            ref =seg.anatomicalFrame
        else:
            ref = seg.getReferential("TF")

        for i in range(0,acqui.GetPointFrameNumber()):
            valX[i,:]= np.dot(ref.motion[i].getRotation() , np.array([100.0,0.0,0.0])) + ref.motion[i].getTranslation()
            valY[i,:]= np.dot(ref.motion[i].getRotation() , np.array([0.0,100.0,0.0])) + ref.motion[i].getTranslation()
            valZ[i,:]= np.dot(ref.motion[i].getRotation() , np.array([0.0,0.0,100.0])) + ref.motion[i].getTranslation()

        btkTools.smartAppendPoint(acqui,targetPointLabel+"_X",valX,desc="")
        btkTools.smartAppendPoint(acqui,targetPointLabel+"_Y",valY,desc="")
        btkTools.smartAppendPoint(acqui,targetPointLabel+"_Z",valZ,desc="")


    def displayMotionViconCoordinateSystem(self,acqui,  segmentLabel,targetPointLabelO,targetPointLabelX,targetPointLabelY,targetPointLabelZ, referential = "Anatomic" ):
        seg=self.getSegment(segmentLabel)

        origin=np.zeros((acqui.GetPointFrameNumber(),3))
        valX=np.zeros((acqui.GetPointFrameNumber(),3))
        valY=np.zeros((acqui.GetPointFrameNumber(),3))
        valZ=np.zeros((acqui.GetPointFrameNumber(),3))


        if referential == "Anatomic":
            ref =seg.anatomicalFrame
        else:
            ref = seg.getReferential("TF")

        for i in range(0,acqui.GetPointFrameNumber()):
            origin[i,:] = ref.motion[i].getTranslation()
            valX[i,:]= np.dot(ref.motion[i].getRotation() , np.array([100.0,0.0,0.0])) + ref.motion[i].getTranslation()
            valY[i,:]= np.dot(ref.motion[i].getRotation() , np.array([0.0,100.0,0.0])) + ref.motion[i].getTranslation()
            valZ[i,:]= np.dot(ref.motion[i].getRotation() , np.array([0.0,0.0,100.0])) + ref.motion[i].getTranslation()

        btkTools.smartAppendPoint(acqui,targetPointLabelO,origin,desc="")
        btkTools.smartAppendPoint(acqui,targetPointLabelX,valX,desc="")
        btkTools.smartAppendPoint(acqui,targetPointLabelY,valY,desc="")
        btkTools.smartAppendPoint(acqui,targetPointLabelZ,valZ,desc="")

    def setClinicalDescriptor(self,jointOrSegmentLabel,dataType, indexes,coefficients, offsets,**options):


        descriptor = ClinicalDescriptor(dataType, jointOrSegmentLabel, indexes,coefficients, offsets,**options)
        self.m_clinicalDescriptors.append(descriptor)

    def getClinicalDescriptor(self,dataType,jointOrSegmentLabel,projection=None):

        if self.m_clinicalDescriptors !=[]:
            for descriptor in self.m_clinicalDescriptors:
                if projection is None:
                    if descriptor.type == dataType and descriptor.label ==jointOrSegmentLabel:
                        infos= descriptor.infos
                        break
                    else:
                        infos = False
                else:
                    if descriptor.type == dataType and descriptor.label ==jointOrSegmentLabel and descriptor.projection == projection:
                        infos= descriptor.infos
                        break
                    else:
                        infos = False
        else:
            infos=False

        if not infos:
            logging.debug("[pyCGM2] : descriptor [ type: %s - label: %s]  not found" %(dataType.name,jointOrSegmentLabel))

        return infos

    def setCoordinateSystemDefinition(self,segmentLabel,coordinateSystemLabel, referentialType):
        dic = {"segmentLabel": segmentLabel,"coordinateSystemLabel": coordinateSystemLabel,"referentialType": referentialType}
        self.m_csDefinitions.append(dic)


class Model6Dof(Model):
    """

    """
    def __init__(self):
        super(Model6Dof, self).__init__()

    def _calibrateTechnicalSegment(self,aquiStatic, segName, dictRef,frameInit,frameEnd, options=None):

        segPicked=self.getSegment(segName)
        for tfName in dictRef[segName]: # TF name

            pt1=aquiStatic.GetPoint(str(dictRef[segName][tfName]['labels'][0])).GetValues()[frameInit:frameEnd,:].mean(axis=0)
            pt2=aquiStatic.GetPoint(str(dictRef[segName][tfName]['labels'][1])).GetValues()[frameInit:frameEnd,:].mean(axis=0)
            pt3=aquiStatic.GetPoint(str(dictRef[segName][tfName]['labels'][2])).GetValues()[frameInit:frameEnd,:].mean(axis=0)

            ptOrigin=aquiStatic.GetPoint(str(dictRef[segName][tfName]['labels'][3])).GetValues()[frameInit:frameEnd,:].mean(axis=0)


            a1=(pt2-pt1)
            a1=np.nan_to_num(np.divide(a1,np.linalg.norm(a1)))

            v=(pt3-pt1)
            v=np.nan_to_num(np.divide(v,np.linalg.norm(v)))

            a2=np.cross(a1,v)
            a2=np.nan_to_num(np.divide(a2,np.linalg.norm(a2)))

            x,y,z,R=frame.setFrameData(a1,a2,dictRef[segName][tfName]['sequence'])

            segPicked.referentials[-1].static.m_axisX=x # work on the last TF in the list : thus index -1
            segPicked.referentials[-1].static.m_axisY=y
            segPicked.referentials[-1].static.m_axisZ=z

            segPicked.referentials[-1].static.setRotation(R)
            segPicked.referentials[-1].static.setTranslation(ptOrigin)

            #  - add Nodes in segmental static(technical)Frame -
            for label in segPicked.m_markerLabels:
                globalPosition=aquiStatic.GetPoint(str(label)).GetValues()[frameInit:frameEnd,:].mean(axis=0)
                segPicked.referentials[-1].static.addNode(label,globalPosition,positionType="Global")

    def _calibrateAnatomicalSegment(self,aquiStatic, segName, dictAnatomic,frameInit,frameEnd, options=None):

        # calibration of technical Frames
        for segName in dictAnatomic:

            segPicked=self.getSegment(segName)
            tf=segPicked.getReferential("TF")

            nd1 = str(dictAnatomic[segName]['labels'][0])
            pt1 = tf.static.getNode_byLabel(nd1).m_global

            nd2 = str(dictAnatomic[segName]['labels'][1])
            pt2 = tf.static.getNode_byLabel(nd2).m_global

            nd3 = str(dictAnatomic[segName]['labels'][2])
            pt3 = tf.static.getNode_byLabel(nd3).m_global

            ndO = str(dictAnatomic[segName]['labels'][3])
            ptO = tf.static.getNode_byLabel(ndO).m_global

            a1=(pt2-pt1)
            a1=np.nan_to_num(np.divide(a1,np.linalg.norm(a1)))

            v=(pt3-pt1)
            v=np.nan_to_num(np.divide(v,np.linalg.norm(v)))

            a2=np.cross(a1,v)
            a2=np.nan_to_num(np.divide(a2,np.linalg.norm(a2)))

            x,y,z,R=frame.setFrameData(a1,a2,dictAnatomic[segName]['sequence'])

            segPicked.anatomicalFrame.static.m_axisX=x # work on the last TF in the list : thus index -1
            segPicked.anatomicalFrame.static.m_axisY=y
            segPicked.anatomicalFrame.static.m_axisZ=z

            segPicked.anatomicalFrame.static.setRotation(R)
            segPicked.anatomicalFrame.static.setTranslation(ptO)

            # --- relative rotation Technical Anatomical
            tf.setRelativeMatrixAnatomic( np.dot(tf.static.getRotation().T,segPicked.anatomicalFrame.static.getRotation()))

            # add tracking markers as node
            for label in segPicked.m_markerLabels:
                globalPosition=aquiStatic.GetPoint(str(label)).GetValues()[frameInit:frameEnd,:].mean(axis=0)
                segPicked.anatomicalFrame.static.addNode(label,globalPosition,positionType="Global")


    def computeMotionTechnicalFrame(self,aqui,segName,dictRef,method,options=None):
        segPicked=self.getSegment(segName)
        segPicked.getReferential("TF").motion =[]
        if method == enums.motionMethod.Sodervisk :
            tms= segPicked.m_tracking_markers
            for i in range(0,aqui.GetPointFrameNumber()):
                visibleMarkers = btkTools.getVisibleMarkersAtFrame(aqui,tms,i)

                # constructuion of the input of sodervisk
                arrayStatic = np.zeros((len(visibleMarkers),3))
                arrayDynamic = np.zeros((len(visibleMarkers),3))

                j=0
                for vm in visibleMarkers:
                    arrayStatic[j,:] = segPicked.getReferential("TF").static.getNode_byLabel(vm).m_global
                    arrayDynamic[j,:] = aqui.GetPoint(vm).GetValues()[i,:]
                    j+=1

                Ropt, Lopt, RMSE, Am, Bm=motion.segmentalLeastSquare(arrayStatic,arrayDynamic)
                R=np.dot(Ropt,segPicked.getReferential("TF").static.getRotation())
                tOri=np.dot(Ropt,segPicked.getReferential("TF").static.getTranslation())+Lopt

                cframe=frame.Frame()
                cframe.setRotation(R)
                cframe.setTranslation(tOri)
                cframe.m_axisX=R[:,0]
                cframe.m_axisY=R[:,1]
                cframe.m_axisZ=R[:,2]

                segPicked.getReferential("TF").addMotionFrame(copy.deepcopy(cframe) )
        else:
            raise Exception("[pyCGM2] : motion method doesn t exist")

    def computeMotionAnatomicalFrame(self,aqui,segName,dictAnatomic,options=None):

        segPicked=self.getSegment(segName)

        segPicked.anatomicalFrame.motion=[]

        ndO = str(dictAnatomic[segName]['labels'][3])
        ptO = segPicked.getReferential("TF").getNodeTrajectory(ndO)

        csFrame=frame.Frame()
        for i in range(0,aqui.GetPointFrameNumber()):
            R = np.dot(segPicked.getReferential("TF").motion[i].getRotation(), segPicked.getReferential("TF").relativeMatrixAnatomic)
            csFrame.update(R,ptO)
            segPicked.anatomicalFrame.addMotionFrame(copy.deepcopy(csFrame))


# --------  MODEL COMPONANTS ---------
class Referential(object):
    """
         A `Referential` is a coordinate system
    """
    def __init__(self):
        self.static=frame.Frame()
        self.motion=[]
        self.relativeMatrixAnatomic = np.zeros((3,3))
        self.additionalInfos = dict()

    def setStaticFrame(self,Frame):
        """
            Set a `Frame` to the member Static of the `Referential`

            :Parameters:
                - `Frame` (pyCGM2.Model.CGM2.frame.Frame) - pyCGM2-Frame instance
        """
        self.static = Frame


    def addMotionFrame(self,Frame):
        """
             Add a `Frame` to the motion member of the `Referential`

            :Parameters:
                - `Frame` (pyCGM2.Model.CGM2.frame.Frame) - pyCGM2-Frame instance

        """
        self.motion.append(Frame)

    def getNodeTrajectory(self,label):
        """
            Get trajectory of a node

            :Parameters:
                - `label` (str) - label of the desired node

            :Return:
                - `pt` (numpy.array(:,3)) - values of the global point trajectory

        """

        node=self.static.getNode_byLabel(label)
        nFrames=len(self.motion)
        pt=np.zeros((nFrames,3))
        for i in range(0,nFrames):
            pt[i,:]=np.dot(self.motion[i].getRotation(),node.m_local)+ self.motion[i].getTranslation()

        return pt



class TechnicalReferential(Referential):
    """
        A `TechnicalReferential` inherit from `Referential`. This specification defines a *Technical* Coordinate system of the segment
    """
    def __init__(self, label):
        """
            :Parameters:
                - `label` (str) - label of the technical referential
        """
        #TODO - By labelling a technical referential, the code accepts multi-technical referentials per Segment. This feature should be removed


        super(TechnicalReferential, self).__init__()


        self.label=label
        self.relativeMatrixAnatomic=np.eye(3,3)


    def setRelativeMatrixAnatomic(self, array):
        """
            Set the relative rigid rotation of the anatomical Referential expressed in the technical referential (:math:`R^a_t`).

            :Parameters:
                - `array` (numpy.array(3,3) - rigid rotation
        """
        self.relativeMatrixAnatomic = array

class AnatomicalReferential(Referential):
    """
        A `AnatomicalReferential` inherit from `Referential`. This specification defines the *Anatomical* Coordinate system of a segment
    """
    def __init__(self):
        super(AnatomicalReferential, self).__init__()





class Segment(object):
    """
        A Segment defines a solid composing of the model
    """
    #TODO
    #       Referential doesn't distinguish anatomical frame  from technical frame
    #       used only for tracking gait and/or functional calibration
    #
    #       compute constant matrix rotation between each referential.static


    def __init__(self,label,index,sideEnum,calibration_markers=[], tracking_markers = []):
        """
            :Parameters:
                - `label` (str) - label of the segment
                - `index` (str) - index of the segment
                - `sideEnum` (pyCGM2.enums) - body side
                - `lst_markerlabel` (list of str) - sequence angle
                - `tracking_markers` (list of str) - sequence angle
        """

        self.name=label
        self.index=index
        self.side = sideEnum

        self.m_tracking_markers=tracking_markers
        self.m_calibration_markers = calibration_markers

        self.m_markerLabels=calibration_markers+tracking_markers
        self.referentials=[]
        self.anatomicalFrame =AnatomicalReferential()

        self.m_bsp = dict()
        self.m_bsp["mass"] = 0
        self.m_bsp["length"] = 0
        self.m_bsp["rog"] = 0
        self.m_bsp["com"] = np.zeros((3))
        self.m_bsp["inertia"] = np.zeros((3,3))

        self.m_externalDeviceWrenchs = list()
        self.m_externalDeviceBtkWrench = None
        self.m_proximalWrench = None

        self.m_proximalMomentContribution = dict()
        self.m_proximalMomentContribution["internal"] = None
        self.m_proximalMomentContribution["external"] = None
        self.m_proximalMomentContribution["inertia"] = None
        self.m_proximalMomentContribution["linearAcceleration"] = None
        self.m_proximalMomentContribution["gravity"] = None
        self.m_proximalMomentContribution["externalDevices"] = None
        self.m_proximalMomentContribution["distalSegments"] = None
        self.m_proximalMomentContribution["distalSegmentForces"] = None
        self.m_proximalMomentContribution["distalSegmentMoments"] = None

        self.m_info = dict()
        self.m_isCloneOf = False

    def removeTrackingMarker(self,label):
        """
            Add a tracking marker

            :Parameters:
                - `label` (str) - marker label
        """
        if label in self.m_tracking_markers:
            self.m_tracking_markers.remove(label)
            self.m_markerLabels.remove(label)
        else:
            logging.debug("tracking marker %s  remove" % label)


    def addTrackingMarkerLabel(self,label):
        """
            Add a tracking marker

            :Parameters:
                - `label` (str) - marker label
        """
        if label not in self.m_tracking_markers:
            self.m_tracking_markers.append(label)
            self.m_markerLabels.append(label)
        else:
            logging.debug("marker %s already in the tracking marker segment list" % label)

    def addCalibrationMarkerLabel(self,label):
        """
            Add a calibration marker

            :Parameters:
                - `label` (str) - marker label
        """

        if label not in self.m_calibration_markers:
            self.m_calibration_markers.append(label)
            self.m_markerLabels.append(label)
        else:
            logging.debug("marker %s already in the clibration marker segment list" % label)


    def resetMarkerLabels(self):
        self.m_markerLabels = list()
        self.m_markerLabels = self.m_tracking_markers + self.m_calibration_markers


    def addMarkerLabel(self,label):
        """
            Add a marker

            :Parameters:
                - `label` (str) - marker label
        """
        isFind=False
        i=0
        for marker in self.m_markerLabels:
            if label in marker:
                isFind=True
                index = i
            i+=1

        if isFind:
            logging.debug("marker %s already in the marker segment list" % label)


        else:
            self.m_markerLabels.append(label)

    def zeroingExternalDevice(self):
        """
            Zeroing external device wrench
        """
        self.m_externalDeviceWrenchs = list()
        self.m_externalDeviceBtkWrench = None

    def zeroingProximalWrench(self):
        """
            Zeroing proximal wrench
        """
        self.m_proximalWrench = None


    def downSampleExternalDeviceWrenchs(self,appf):
        """
            Downsample external device wrenchs

            :Parameters:
                - `appf` (int) - analog point per frame
        """
        if self.isExternalDeviceWrenchsConnected():

            for wrIt in  self.m_externalDeviceWrenchs:
                forceValues = wrIt.GetForce().GetValues()
                forceValues_ds= forceValues[::appf]
                wrIt.GetForce().SetValues(forceValues_ds)

                momentValues = wrIt.GetMoment().GetValues()
                momentValues_ds= momentValues[::appf]
                wrIt.GetMoment().SetValues(momentValues_ds)

                positionValues = wrIt.GetPosition().GetValues()
                positionValues_ds= positionValues[::appf]
                wrIt.GetPosition().SetValues(positionValues_ds)


    def isExternalDeviceWrenchsConnected(self):
        """
            Detect external device wrenchs
        """
        if self.m_externalDeviceWrenchs == []:
            return False
        else:
            return True

    def addExternalDeviceWrench(self, btkWrench):
        """
            Add an external device wrench

            :Parameters:
                - `btkWrench` (btk.wrench) - a btk wrench instance

        """
        self.m_externalDeviceWrenchs.append(btkWrench)



    def setMass(self, value):
        """
            Set segment mass

            :Parameters:
                - `value` (float) - mass

        """
        self.m_bsp["mass"] = value

    def setLength(self, value):
        """
            Set segment length

            :Parameters:
                - `value` (float) - length

        """
        self.m_bsp["length"] = value

    def setRog(self, value):
        """
            Set segment radius of giration

            :Parameters:
                - `value` (float) - radius of giration

        """
        self.m_bsp["rog"] = value

    def setComPosition(self, array3):
        """
            Set local position of the centre of mass

            :Parameters:
                - `array3` (numpy.array(3,)) - centre of mass position

        """
        self.m_bsp["com"] = array3

    def setInertiaTensor(self, array33):
        """
            Set segment inertia tensor

            :Parameters:
                - `array33` (numpy.array(3,3)) - tensor of inertia

        """
        self.m_bsp["inertia"] = array33










    def addTechnicalReferential(self,label):
        """
            Add a technical referential

            :Parameters:
                - `label` (str) - technical frame label
        """


        ref=TechnicalReferential(label)
        self.referentials.append(ref)



    def getReferential(self,label):
        """
            Access a referential from its label

            :Parameters:
                - `label` (str) - technical referential label

        """

        for tfIt in  self.referentials:
            if tfIt.label == label:
                return tfIt



    def getComTrajectory(self,exportBtkPoint=False, btkAcq=None):
        """
            Get global trajectory of the centre of mass

            :Parameters:
                - `exportBtkPoint` (bool) - enable export as btk point
                - `btkAcq` (btk acquisition) - a btk acquisition instance

            :Return:
                - `values` (numpy.array(n,3)) - values of the com trajectory
        """

        frameNumber = len(self.anatomicalFrame.motion)
        values = np.zeros((frameNumber,3))
        for i in range(0,frameNumber):
            values[i,:] = np.dot(self.anatomicalFrame.motion[i].getRotation() ,self.m_bsp["com"]) + self.anatomicalFrame.motion[i].getTranslation()

        if exportBtkPoint:
            if btkAcq != None:
                btkTools.smartAppendPoint(btkAcq,self.name + "_com",values,desc="com")
        return values


    def getComVelocity(self,pointFrequency,method = "spline"):
        """
            Get global linear velocity of the centre of mass

            :Parameters:
                - `pointFrequency` (double) - point frequency
                - `method` (str) - derivation method (spline, spline fitting)

            :Return:
                - `values` (numpy.array(n,3)) - values of the com velocity

            .. note :: if method doesnt recognize, numerical first order derivation is used

        """

        if method == "spline":
            values = derivation.splineDerivation(self.getComTrajectory(),pointFrequency,order=1)
        elif method == "spline fitting":
            values = derivation.splineFittingDerivation(self.getComTrajectory(),pointFrequency,order=1)
        else:
            values = derivation.firstOrderFiniteDifference(self.getComTrajectory(),pointFrequency)

        return values


    def getComAcceleration(self,pointFrequency,method = "spline", **options):
        """
            Get global linear acceleration of the centre of mass

            :Parameters:
                - `pointFrequency` (double) - point frequency
                - `method` (str) - derivation method (spline, spline fitting)
                - `**options` (kwargs) - options pass to method.

            :Return:
                - `values` (numpy.array(n,3)) - values of the com acceleration

            .. attention :: com trajectory can be smoothed  by calling fc and order, the cut-off frequency and the order of the low-pass filter respectively


            .. note ::

                if method doesnt recognize, numerical second order derivation is used


        """

        valueCom = self.getComTrajectory()
        if "fc" in options.keys() and  "order" in options.keys():
            valueCom = signal_processing.arrayLowPassFiltering(valueCom,pointFrequency,options["order"],options["fc"]  )

        if method == "spline":
            values = derivation.splineDerivation(valueCom,pointFrequency,order=2)
        elif method == "spline fitting":
            values = derivation.splineFittingDerivation(self.getComTrajectory(),pointFrequency,order=2)
        else:
            values = derivation.secondOrderFiniteDifference(valueCom,pointFrequency)

        return values





    def getAngularVelocity(self,sampleFrequency,method="conventional"):
        """
            Get angular velocity

            :Parameters:
                - `sampleFrequency` (double) - point frequency
                - `method` (str) - method used for computing the angular velocity

            :Return:
                - `AngularVelocValues` (numpy.array(n,3)) - values of the angular velocity

            .. note::

                The *pig* method reproduce a  bodybuilder code of the plug-in gait.

                .. raw:: python

                      AngVel = [ dot( NextPosR(:,2), PrevPosR(:,3) ); ...
                                 dot( NextPosR(:,3), PrevPosR(:,1) ); ...
                                 dot( NextPosR(:,1), PrevPosR(:,2) ); ...
                               ] ./ (2*SamplePeriod);

                The *conventional* method computes angular velocity through this statement :math:`\dot{R}R^t`

        """


        frameNumber = len(self.anatomicalFrame.motion)
        AngularVelocValues = np.zeros((frameNumber,3))

        # pig method0
        if method == "pig":
            for i in range(1,frameNumber-1):
                omegaX=(np.dot( self.anatomicalFrame.motion[i+1].m_axisY,
                                          self.anatomicalFrame.motion[i-1].m_axisZ))/(2*1/sampleFrequency)

                omegaY=(np.dot( self.anatomicalFrame.motion[i+1].m_axisZ,
                                          self.anatomicalFrame.motion[i-1].m_axisX))/(2*1/sampleFrequency)

                omegaZ=(np.dot( self.anatomicalFrame.motion[i+1].m_axisX,
                                self.anatomicalFrame.motion[i-1].m_axisY))/(2*1/sampleFrequency)

                omega = np.array([[omegaX,omegaY,omegaZ]]).transpose()

                AngularVelocValues[i,:] = np.dot(self.anatomicalFrame.motion[i].getRotation(),  omega).transpose()

        # conventional method
        if method == "conventional":
            rdot = derivation.matrixFirstDerivation(self.anatomicalFrame.motion, sampleFrequency)
            for i in range(1,frameNumber-1):
                tmp =np.dot(rdot[i],self.anatomicalFrame.motion[i].getRotation().transpose())
                AngularVelocValues[i,0]=tmp[2,1]
                AngularVelocValues[i,1]=tmp[0,2]
                AngularVelocValues[i,2]=tmp[1,0]

        return AngularVelocValues


    def getAngularAcceleration(self,sampleFrequency):
        """
            Get angular acceleration

            :Parameters:
                - `sampleFrequency` (double) - point frequency

            :Return:
                - `values` (numpy.array(n,3)) - values of the angular accelration


            .. note:: A first order differention of the angular velocity is used
        """

        values = derivation.firstOrderFiniteDifference(self.getAngularVelocity(sampleFrequency),sampleFrequency)
        return values

class Joint(object):
    """
        a Joint defines relative motion between a proximal and a distal segment
    """

    def __init__(self, label, proxLabel,distLabel,sequence,nodeLabel):
        """
            :Parameters:
                - `label` (str) - label of the chain
                - `proxLabel` (str) - label of the proximal segment
                - `distLabel` (str) - label of the distal segment
                - `sequence` (str) - sequence angle
        """
        self.m_label=label
        self.m_proximalLabel=proxLabel
        self.m_distalLabel=distLabel
        self.m_sequence=sequence
        self.m_nodeLabel=nodeLabel



# -*- coding: utf-8 -*-
import sys
import os
import string

def scanViconTemplatePipeline(sourcePath,desPath,pyCGM2nexusAppsPath):

    toreplace= "C:/Users/HLS501/Documents/Programming/API/pyCGM2/pyCGM2/Apps"

    sourcePath = sourcePath[:-1] if sourcePath[-1:]=="\\" else sourcePath
    desPath = desPath[:-1] if desPath[-1:]=="\\" else desPath
    pyCGM2nexusAppsPath = pyCGM2nexusAppsPath[:-1] if pyCGM2nexusAppsPath[-1:]=="\\" else pyCGM2nexusAppsPath

    pyCGM2nexusAppsPath_antislash = string.replace(pyCGM2nexusAppsPath, '\\', '/')

    for file in os.listdir(sourcePath):
        with open(sourcePath+"\\"+file, 'r') as f:
            file_contents = f.read()

        content = string.replace(file_contents, toreplace,pyCGM2nexusAppsPath_antislash)


        if not os.path.isfile( desPath +"\\"+ file):
            with open(desPath + "\\"+file, "w") as text_file:
                text_file.write(content)



# -*- coding: utf-8 -*-
#import ipdb
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import ipdb


def plotCurve(ax, x,mean,sd, ofo,ofs,ifs):
    ax.plot(x , mean )
    ax.fill_between(x, mean-sd/2.0, mean+sd/2.0,
                    facecolor="blue", alpha=0.5,linewidth=0)

    ax.set_xlim(0,1)

    ax.axvline(ofo,color="black",ls='dashed')
    ax.axvline(ofs,color="black",ls='dashed')
    ax.axvline(ifs,color="black",ls='dashed')



if __name__ == "__main__":

    plt.close("all")
    xls = pd.ExcelFile("Formatted- Schwartz2008.xlsx")
    jointRotations = xls.parse("Joint Rotations")
    jointMoments = xls.parse("Joint Moments")
    jointPower = xls.parse("Joint Power")

    cycleEvents = xls.parse("Cycle Events")

    # free events
    ofo = cycleEvents[ cycleEvents["Parameter"] == "Opposite Foot Off  [% cycle]"]["FreeMean"].values[0]
    ofs = cycleEvents[ cycleEvents["Parameter"] == "Opposite Foot Contact  [% cycle]"]["FreeMean"].values[0]
    ifs = cycleEvents[ cycleEvents["Parameter"] == "Ipsilateral Foot Off  [% cycle]"]["FreeMean"].values[0]

    # ----- Kinematic -----
    data0 = jointRotations [ jointRotations["Angle"] == "Pelvic Ant/Posterior Tilt"]
    data1 = jointRotations [ jointRotations["Angle"] == "Pelvic Up/Down Obliquity"]
    data2 = jointRotations [ jointRotations["Angle"] == "Pelvic Int/External Rotation"]

    data3 = jointRotations [ jointRotations["Angle"] == "Hip Flex/Extension"]
    data4 = jointRotations [ jointRotations["Angle"] == "Hip Ad/Abduction"]
    data5 = jointRotations [ jointRotations["Angle"] == "Hip Int/External Rotation"]

    data6 = jointRotations [ jointRotations["Angle"] == "Knee Flex/Extension"]
    data7 = jointRotations [ jointRotations["Angle"] == "Knee Ad/Abduction"]
    data8 = jointRotations [ jointRotations["Angle"] == "Knee Int/External Rotation"]

    data9 = jointRotations [ jointRotations["Angle"] == "Ankle Dorsi/Plantarflexion"]

    data11 = jointRotations [ jointRotations["Angle"] == "Foot Int/External Progression"]


    fig= plt.figure()
    ax = fig.add_subplot(1,1,1)
    plotCurve(ax, data6["PercentageGaitCycle"],data6["FreeMean"],data6["FreeSd"], ofo,ofs,ifs)

    fig= plt.figure()
    ax = fig.add_subplot(1,1,1)
    plotCurve(ax, data9["PercentageGaitCycle"],data9["FreeMean"],data9["FreeSd"], ofo,ofs,ifs)

    begin = int(10/2.0+1)
    end = int(46/2.0+1)
    di = np.diff(data9["FreeMean"].values[begin:end])
    plt.figure()
    plt.plot(data9["FreeMean"].values[begin:end])
    plt.plot(di)


    slopeMax= (( data9["FreeMean"].values[end]    +data9["FreeSd"].values[end])
              - (data9["FreeMean"].values[begin]  -data9["FreeSd"].values[begin]))/(end-begin)
    slopeMin= (( data9["FreeMean"].values[end]    -data9["FreeSd"].values[end])
              - (data9["FreeMean"].values[begin]  +data9["FreeSd"].values[begin]))/(end-begin)




    # splope of dosiflexion 9 ds1 - startpushoof


    fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
    title=u""" Descriptive Time-normalized Kinematics \n """
    fig.suptitle(title)
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

    ax0 = plt.subplot(5,3,1)# Pelvis X
    ax1 = plt.subplot(5,3,2)# Pelvis Y
    ax2 = plt.subplot(5,3,3)# Pelvis Z
    ax3 = plt.subplot(5,3,4)# Hip X
    ax4 = plt.subplot(5,3,5)# Hip Y
    ax5 = plt.subplot(5,3,6)# Hip Z
    ax6 = plt.subplot(5,3,7)# Knee X
    ax7 = plt.subplot(5,3,8)# Knee Y
    ax8 = plt.subplot(5,3,9)# Knee Z
    ax9 = plt.subplot(5,3,10)# Ankle X
    ax10 = plt.subplot(5,3,11)# Ankle Z
    ax11 = plt.subplot(5,3,12)# Footprogress Z

    ax0.set_title("Pelvis Tilt" ,size=8)
    ax1.set_title("Pelvis Obliquity" ,size=8)
    ax2.set_title("Pelvis Rotation" ,size=8)
    ax3.set_title("Hip Flexion" ,size=8)
    ax4.set_title("Hip Adduction" ,size=8)
    ax5.set_title("Hip Rotation" ,size=8)
    ax6.set_title("Knee Flexion" ,size=8)
    ax7.set_title("Knee Adduction" ,size=8)
    ax8.set_title("Knee Rotation" ,size=8)
    ax9.set_title("Ankle dorsiflexion" ,size=8)
    ax10.set_title("Ankle eversion" ,size=8)
    ax11.set_title("Foot Progression " ,size=8)


    plotCurve(ax0, data0["PercentageGaitCycle"],data0["FreeMean"],data0["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax1, data1["PercentageGaitCycle"],data1["FreeMean"],data1["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax2, data2["PercentageGaitCycle"],data2["FreeMean"],data2["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax3, data3["PercentageGaitCycle"],data3["FreeMean"],data3["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax4, data4["PercentageGaitCycle"],data4["FreeMean"],data4["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax5, data5["PercentageGaitCycle"],data5["FreeMean"],data5["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax6, data6["PercentageGaitCycle"],data6["FreeMean"],data6["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax7, data7["PercentageGaitCycle"],data7["FreeMean"],data7["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax8, data8["PercentageGaitCycle"],data8["FreeMean"],data8["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax9, data9["PercentageGaitCycle"],data9["FreeMean"],data9["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax11, data11["PercentageGaitCycle"],data11["FreeMean"],data11["FreeSd"], ofo,ofs,ifs)




    # ----- Kinetics -----
    fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
    title=u""" Descriptive Time-normalized Kinetics \n """
    fig.suptitle(title)
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

    ax0 = plt.subplot(3,4,1)# Hip X extensor
    ax1 = plt.subplot(3,4,2)# Hip Y abductor
    ax2 = plt.subplot(3,4,3)# Hip Z rotation
    ax3 = plt.subplot(3,4,4)# Knee Z power

    ax4 = plt.subplot(3,4,5)# knee X extensor
    ax5 = plt.subplot(3,4,6)# knee Y abductor
    ax6 = plt.subplot(3,4,7)# knee Z rotation
    ax7 = plt.subplot(3,4,8)# knee Z power

    ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
    ax9 = plt.subplot(3,4,10)# ankle Y rotation
    ax10 = plt.subplot(3,4,11)# ankle Z everter
    ax11 = plt.subplot(3,4,12)# ankle Z power

    ax0.set_title("Hip extensor Moment" ,size=8)
    ax1.set_title("Hip abductor Moment" ,size=8)
    ax2.set_title("Hip rotation Moment" ,size=8)
    ax3.set_title("Hip Power" ,size=8)

    ax4.set_title("Knee extensor Moment" ,size=8)
    ax5.set_title("Knee abductor Moment" ,size=8)
    ax6.set_title("Knee rotation Moment" ,size=8)
    ax7.set_title("Knee Power" ,size=8)

    ax8.set_title("Ankle plantarflexor Moment" ,size=8)
    ax9.set_title("Ankle everter Moment" ,size=8)
    ax10.set_title("Ankle abductor Moment" ,size=8)
    ax11.set_title("Ankle Power " ,size=8)

    data0 = jointMoments [ jointMoments["Moment"] == "Hip Ext/Flexion"]
    data1 = jointMoments [ jointMoments["Moment"] == "Hip Ab/Adduction"]
    # no Int-rot
    data3 = jointPower [ jointPower["Power"] == "Hip"]

    data4= jointMoments [ jointMoments["Moment"] == "Knee Ext/Flexion"]
    data5 = jointMoments [ jointMoments["Moment"] == "Knee Ab/Adduction"]
    # no 6
    data7 = jointPower [ jointPower["Power"] == "Knee"]

    data8 = jointMoments [ jointMoments["Moment"] == "Ankle Dorsi/Plantarflexion"]
    #no 9
    #no 10
    data11 = jointPower [ jointPower["Power"] == "Ankle"]


    plotCurve(ax0, data0["PercentageGaitCycle"],data0["FreeMean"],data0["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax1, data1["PercentageGaitCycle"],data1["FreeMean"],data1["FreeSd"], ofo,ofs,ifs)
    # no2
    plotCurve(ax3, data3["PercentageGaitCycle"],data3["FreeMean"],data3["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax4, data4["PercentageGaitCycle"],data4["FreeMean"],data4["FreeSd"], ofo,ofs,ifs)
    plotCurve(ax5, data5["PercentageGaitCycle"],data5["FreeMean"],data5["FreeSd"], ofo,ofs,ifs)
    # no6
    plotCurve(ax7, data7["PercentageGaitCycle"],data7["FreeMean"],data7["FreeSd"], ofo,ofs,ifs)

    plotCurve(ax8, data8["PercentageGaitCycle"],data8["FreeMean"],data8["FreeSd"], ofo,ofs,ifs)
    # no9
    # no 10
    plotCurve(ax11, data11["PercentageGaitCycle"],data11["FreeMean"],data11["FreeSd"], ofo,ofs,ifs)




    plt.show()



# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import logging
import matplotlib.pyplot as plt
from collections import OrderedDict

from pyCGM2.Tools import exportTools
from pyCGM2.Signal.detect_peaks import detect_peaks
from pyCGM2.Math import derivation

from pyCGM2.Processing import analysisHandler
from pyCGM2.Tools import exportTools
# --- FILTER ----


class DiscreteEMGFilter(object):
    """
    """

    def __init__(self, discreteEMGProcedure, analysis,emgLabels, emgMuscles, emgContexts,subjInfo=None,condExpInfo=None):

        self.m_procedure = discreteEMGProcedure
        self.m_analysis=analysis
        self.dataframe =  None

        self.m_subjInfo = subjInfo
        self.m_condExpInfo = condExpInfo

        self.m_emgMuscles=emgMuscles
        self.m_emgLabels=emgLabels
        self.m_emgContexts=emgContexts

    def setSubjInfo(self, subjInfo):
        self.m_subjInfo = subjInfo

    def setCondExpInf(self, condExpInfo):
        self.m_condExpInfo = condExpInfo


    def getOutput(self):
        self.dataframe = self.m_procedure.detect(self.m_analysis,self.m_emgLabels,self.m_emgMuscles,self.m_emgContexts)

        # add infos

        if self.m_subjInfo is not None:
            for key,value in self.m_subjInfo.items():
                exportTools.isColumnNameExist( self.dataframe, key)
                self.dataframe[key] = value

        if self.m_condExpInfo is not None:
            for key,value in self.m_condExpInfo.items():
                exportTools.isColumnNameExist( self.dataframe, key)
                self.dataframe[key] = value

        return self.dataframe





# --- PROCEDURE ----



class AmplitudesProcedure(object):

    NAME = "EMG Amplitude from Integration for each gait phase"


    def __init__(self):
        pass

    def detect (self,analysisInstance,emgLabels,emgMuscles,emgContexts):

        dataframes = list()

        index = 0
        for emgLabel in emgLabels:
            context = emgContexts[index]
            muscle = emgMuscles[index]
            dataframes.append(self.__getAmplitudebyPhase(analysisInstance,emgLabel+"_Rectify_Env_Norm",muscle,context))
            index+=1

        return pd.concat(dataframes)



    def __construcPandasSerie(self,emgLabel,muscle,context, cycleIndex,phase,
                              value,procedure):
        iDict = OrderedDict([('ChannelLabel', emgLabel),
                     ('Label', muscle),
                     ('EventContext', context),
                     ('Cycle', cycleIndex),
                     ('Phase', phase),
                     ('Procedure', procedure),
                     ('Value', value)])
        return pd.Series(iDict)

    def __getAmplitudebyPhase(self,analysisInstance,emglabel,muscle,context):

        procedure = "Amplitude"
        muscle = context[0]+muscle


        stanceValues =   analysisInstance.emgStats.pst['stancePhase', context]['values']
        doubleStance1Values =   analysisInstance.emgStats.pst['doubleStance1', context]['values']
        doubleStance2Values =   stanceValues - analysisInstance.emgStats.pst['doubleStance2', context]['values']

        normalizedCycleValues = analysisInstance.emgStats.data [emglabel,context]

        series = list()

        for i in range(0,len(normalizedCycleValues["values"])):

            # cycle
            lim0 = 0; lim1 = 100
            res_cycle=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "cycle",
                                               res_cycle,
                                               procedure)
            series.append(serie)

            # stance
            lim0 = 0; lim1 = int(stanceValues[i])
            res_stance=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "stance",
                                               res_stance,
                                               procedure)
            series.append(serie)


            # swing
            lim0 = int(stanceValues[i]); lim1 = 100
            res_swing=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "swing",
                                               res_swing,
                                               procedure)
            series.append(serie)


            # doubleStance1
            lim0 = 0; lim1 = int(doubleStance1Values[i])
            res_d1=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "doubleStance1",
                                               res_d1,
                                               procedure)
            series.append(serie)

            # double stance2
            lim0 = int(doubleStance2Values[i]); lim1 = int(stanceValues[i])
            res_d2=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "doubleStance2",
                                               res_d2,
                                               procedure)
            series.append(serie)

            lim0 = int(doubleStance1Values[i]); lim1 = int(doubleStance2Values[i])
            res_single=np.trapz(normalizedCycleValues["values"][i][lim0:lim1+1],
                         x=np.arange(lim0,lim1+1),axis=0)[0]

            serie = self.__construcPandasSerie(emglabel,muscle,context,
                                               int(i),
                                               "singleStance",
                                               res_single,
                                               procedure)
            series.append(serie)


        return pd.DataFrame(series)



# coding: utf-8
from __future__ import unicode_literals
import pyCGM2
import numpy as np
from scipy import spatial
import logging
import encodings
from pyCGM2 import btk
from pyCGM2.Utils import utils



# --- acquisition -----
def smartReader(filename,translators=None):
    """
        Convenient function to read a c3d with Btk

        :Parameters:
            - `filename` (str) - path and filename of the c3d
            - `translators` (str) - marker translators
    """
    reader = btk.btkAcquisitionFileReader()
    reader.SetFilename(utils.str(filename))
    reader.Update()
    acq=reader.GetOutput()
    if translators is not None:
        acq =  applyTranslators(acq,translators)

    # management force plate type 5
    if checkForcePlateExist(acq):
        if "5" in smartGetMetadata(acq,"FORCE_PLATFORM","TYPE"):
            logging.warning("[pyCGM2] Type 5 Force plate detected. Due to a BTK known-issue,  type 5 force plate has been corrected as type 2")
            from pyCGM2.ForcePlates import forceplates # inelegant code but avoir circular import !!
            forceplates.correctForcePlateType5(acq)

    # sort events
    sortedEvents(acq)

    return acq

def smartWriter(acq, filename):
    """
        Convenient function to write a c3d with Btk

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `filename` (str) - path and filename of the c3d
    """
    writer = btk.btkAcquisitionFileWriter()
    writer.SetInput(acq)
    writer.SetFilename(utils.str(filename))
    writer.Update()




def GetMarkerNames(acq):
    """
    """

    markerNames=[]
    for it in btk.Iterate(acq.GetPoints()):
        if it.GetType() == btk.btkPoint.Marker and it.GetLabel()[0] !="*":
            markerNames.append(it.GetLabel())
    return markerNames


def findNearestMarker(acq,i,marker,markerNames=None):
    values = acq.GetPoint(utils.str(marker)).GetValues()[i,:]

    if markerNames is None:
        markerNames=[]
        for it in btk.Iterate(acq.GetPoints()):
            if it.GetType() == btk.btkPoint.Marker and it.GetLabel()[0] !="*" and it.GetLabel() != utils.str(marker):
                markerNames.append(it.GetLabel())

    j=0
    out = np.zeros((len(markerNames),3))
    for name in markerNames :
        out[j,:] = acq.GetPoint(name).GetValues()[i,:]
        j+=1

    tree = spatial.KDTree(out)
    dist,index = tree.query(values)


    return markerNames[index],dist




def GetAnalogNames(acq):
    analogNames=[]
    for it in btk.Iterate(acq.GetAnalogs()):
        analogNames.append(it.GetLabel())
    return analogNames


def isGap(acq, markerLabel):
    """
        Check if there is a gap

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `markerList` (list of str) - marker labels
    """
    residualValues = acq.GetPoint(utils.str(markerLabel)).GetResiduals()
    if any(residualValues== -1.0):
        logging.warning("[pyCGM2] gap found for marker (%s)"%(markerLabel))
        return True
    else:
        return False

def findMarkerGap(acq):
    gaps = list()
    markerNames  = GetMarkerNames(acq)
    for marker in markerNames:
        if isGap(acq,marker):
            gaps.append(marker)

    return gaps



def isPointExist(acq,label):
    """
        Check if a point label exists inside an acquisition

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `label` (str) - point label
    """
    if acq.GetPointNumber()==0:
        return False
    else:
        i = acq.GetPoints().Begin()
        while i != acq.GetPoints().End():
            if i.value().GetLabel()== utils.str(label):
                flagPoint= True
                break
            else:
                i.incr()
                flagPoint= False

        if flagPoint:
            return True
        else:
            return False

def isPointsExist(acq,labels):
    """
        Check if point labels exist inside an acquisition

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `labels` (list of str) - point labels
    """
    for label in labels:
        if not isPointExist(acq,label):
            logging.warning("[pyCGM2] markers (%s) doesn't exist"% label )
            return False
    return True

def smartAppendPoint(acq,label,values, PointType=btk.btkPoint.Marker,desc="",residuals = None):
    """
        Append/Update a point inside an acquisition

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `label` (str) - point label
            - `values` (numpy.array(n,3)) - point label
            - `PointType` (enums of btk.btkPoint) - type of Point
    """

    logging.debug("new point (%s) added to the c3d" % label)

    # TODO : si value = 1 lignes alors il faudrait dupliquer la lignes pour les n franes
    # valueProj *np.ones((aquiStatic.GetPointFrameNumber(),3))

    values = np.nan_to_num(values)

    if residuals is None:
        residuals = np.zeros((values.shape[0]))
        for i in range(0, values.shape[0]):
            if np.all(values[i,:] == np.zeros((3))):
                residuals[i] = -1
            else:
                residuals[i] = 0

    if isPointExist(acq,label):
        acq.GetPoint(utils.str(label)).SetValues(values)
        acq.GetPoint(utils.str(label)).SetDescription(utils.str(desc))
        acq.GetPoint(utils.str(label)).SetType(PointType)
        acq.GetPoint(utils.str(label)).SetResiduals(residuals)

    else:
        new_btkPoint = btk.btkPoint(utils.str(label),acq.GetPointFrameNumber())
        new_btkPoint.SetValues(values)
        new_btkPoint.SetDescription(utils.str(desc))
        new_btkPoint.SetType(PointType)
        new_btkPoint.SetResiduals(residuals)
        acq.AppendPoint(new_btkPoint)

def clearPoints(acq, pointlabelList):
    """
        Clear points

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `lapointlabelListel` (list of str) - point labels

    """

    i = acq.GetPoints().Begin()
    while i != acq.GetPoints().End():
        label =  i.value().GetLabel()
        if label not in pointlabelList:
            i = acq.RemovePoint(i)
            logging.debug( label + " removed")
        else:
            i.incr()
            logging.debug( label + " found")



    return acq

def checkFirstAndLastFrame (acq, markerLabel):
    """
        Check if extremity frames are correct

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `markerLabel` (str) - marker label
    """

    if acq.GetPoint(utils.str(markerLabel)).GetValues()[0,0] == 0:
        raise Exception ("[pyCGM2] no marker on first frame")

    if acq.GetPoint(utils.str(markerLabel)).GetValues()[-1,0] == 0:
        raise Exception ("[pyCGM2] no marker on last frame")

def isGap_inAcq(acq, markerList):
    """
        Check if there is a gap

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `markerList` (list of str) - marker labels
    """
    for m in markerList:
         residualValues = acq.GetPoint(utils.str(m)).GetResiduals()
         if any(residualValues== -1.0):
             raise Exception("[pyCGM2] gap founded for markers %s " % m )

def findValidFrames(acq,markerLabels):

    flag = list()
    for i in range(0,acq.GetPointFrameNumber()):
        pointFlag=list()
        for marker in markerLabels:
            if acq.GetPoint(utils.str(marker)).GetResidual(i) >= 0 :
                pointFlag.append(1)
            else:
                pointFlag.append(0)

        if all(pointFlag)==1:
            flag.append(1)
        else:
            flag.append(0)

    firstValidFrame = flag.index(1)
    lastValidFrame = len(flag) - flag[::-1].index(1) - 1

    return flag,firstValidFrame,lastValidFrame


def applyValidFramesOnOutput(acq,validFrames):

    validFrames = np.asarray(validFrames)

    for it in btk.Iterate(acq.GetPoints()):
        if it.GetType() in [btk.btkPoint.Angle, btk.btkPoint.Force, btk.btkPoint.Moment,btk.btkPoint.Power]:
            values = it.GetValues()
            for i in range(0,3):
                values[:,i] =  values[:,i] * validFrames
            it.SetValues(values)

def checkMultipleSubject(acq):
    if acq.GetPoint(0).GetLabel().count(":"):
        raise Exception("[pyCGM2] Your input static c3d was saved with two activate subject. Re-save it with only one before pyCGM2 calculation")


# --- Model -----

def applyTranslators(acq, translators):
    """
    Rename marker from translators
    :Parameters:
        - `acq` (btkAcquisition) - a btk acquisition instance
        - `translators` (dict) - translators
    """
    acqClone = btk.btkAcquisition.Clone(acq)

    if translators is not None:
        modifiedMarkerList = list()

        # gather all labels
        for it in translators.items():
            wantedLabel,initialLabel = it[0],it[1]
            if wantedLabel != initialLabel and initialLabel !="None":
                modifiedMarkerList.append(it[0])
                modifiedMarkerList.append(it[1])

        # Remove Modified Markers from Clone
        for point in  btk.Iterate(acq.GetPoints()):
            if point.GetType() == btk.btkPoint.Marker:
                label = point.GetLabel()
                if label in modifiedMarkerList:
                    acqClone.RemovePoint(utils.str(label))
                    logging.debug("point (%s) remove in the clone acq  " %((label)))

        # Add Modify markers to clone
        for it in translators.items():
            wantedLabel,initialLabel = it[0],it[1]
            if initialLabel !="None":
                if isPointExist(acq,wantedLabel):
                    smartAppendPoint(acqClone,(wantedLabel+"_origin"),acq.GetPoint(utils.str(wantedLabel)).GetValues(),PointType=btk.btkPoint.Marker) # modified marker
                    logging.warning("wantedLabel (%s)_origin created" %((wantedLabel)))
                if isPointExist(acq,initialLabel):
                    if initialLabel in translators.keys():
                        if translators[initialLabel] == "None":
                            logging.warning("Initial point (%s)and (%s) point to similar values" %((initialLabel), (wantedLabel)))
                            smartAppendPoint(acqClone,(wantedLabel),acq.GetPoint(utils.str(initialLabel)).GetValues(),PointType=btk.btkPoint.Marker)
                            smartAppendPoint(acqClone,(initialLabel),acq.GetPoint(utils.str(initialLabel)).GetValues(),PointType=btk.btkPoint.Marker) # keep initial marker
                        elif translators[initialLabel] == wantedLabel:
                            logging.warning("Initial point (%s) swaped with (%s)" %((initialLabel), (wantedLabel)))
                            initialValue = acq.GetPoint(utils.str(initialLabel)).GetValues()
                            wantedlValue = acq.GetPoint(utils.str(wantedLabel)).GetValues()
                            smartAppendPoint(acqClone,(wantedLabel),initialValue,PointType=btk.btkPoint.Marker)
                            smartAppendPoint(acqClone,("TMP"),wantedlValue,PointType=btk.btkPoint.Marker)
                            acqClone.GetPoint(utils.str("TMP")).SetLabel(initialLabel)
                            acqClone.RemovePoint(utils.str(wantedLabel+"_origin"))
                            acqClone.RemovePoint(utils.str(initialLabel+"_origin"))
                    else:
                        logging.warning("Initial point (%s) renamed (%s)  added into the c3d" %((initialLabel), (wantedLabel)))
                        smartAppendPoint(acqClone,(wantedLabel),acq.GetPoint(utils.str(initialLabel)).GetValues(),PointType=btk.btkPoint.Marker)
                        smartAppendPoint(acqClone,(initialLabel),acq.GetPoint(utils.str(initialLabel)).GetValues(),PointType=btk.btkPoint.Marker)

                else:
                    logging.warning("initialLabel (%s) doesn t exist  " %((initialLabel)))
                    #raise Exception ("your translators are badly configured")


    return acqClone




def checkMarkers( acq, markerList):
    """
        Check if marker labels exist inside an acquisition

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `markerList` (list of str) - marker labels
    """
    for m in markerList:
        if not isPointExist(acq, m):
            raise Exception("[pyCGM2] markers %s not found" % m )



# --- events -----


def clearEvents(acq,labels):

    events= acq.GetEvents()
    newEvents=btk.btkEventCollection()
    for ev in btk.Iterate(events):
        if ev.GetLabel() not in labels:
            newEvents.InsertItem(ev)

    acq.ClearEvents()
    acq.SetEvents(newEvents)



def modifyEventSubject(acq,newSubjectlabel):
    """
        update the subject name of all events

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `newSubjectlabel` (str) - desired subject name
    """

    # events
    nEvents = acq.GetEventNumber()
    if nEvents>=1:
        for i in range(0, nEvents):
            acq.GetEvent(i).SetSubject(utils.str(newSubjectlabel))
    return acq

def modifySubject(acq,newSubjectlabel):
    """
        update the subject name inside c3d metadata

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `newSubjectlabel` (str) - desired subject name
    """
    acq.GetMetaData().FindChild(utils.str("SUBJECTS") ).value().FindChild(utils.str("NAMES")).value().GetInfo().SetValue(0,(utils.str(newSubjectlabel)))


def getNumberOfModelOutputs(acq):
    n_angles=0
    n_forces=0
    n_moments=0
    n_powers=0


    for it in btk.Iterate(acq.GetPoints()):
        if it.GetType() == btk.btkPoint.Angle:
            n_angles+=1

        if it.GetType() == btk.btkPoint.Force:
            n_forces+=1

        if it.GetType() == btk.btkPoint.Moment:
            n_moments+=1

        if it.GetType() == btk.btkPoint.Power:
            n_powers+=1

    return  n_angles,n_forces ,n_moments,n_powers

# --- metadata -----
def hasChild(md,mdLabel):
    """
        Check if a label is within metadata

        .. note::

            btk has a HasChildren method. HasChild doesn t exist, you have to use MetadataIterator to loop metadata

        :Parameters:
            - `md` (btkMetadata) - a btk metadata instance
            - `mdLabel` (str) - label of the metadata you want to check
    """

    outMd = None
    for itMd in btk.Iterate(md):
        if itMd.GetLabel() == utils.str(mdLabel):
            outMd = itMd
            break
    return outMd


def getVisibleMarkersAtFrame(acq,markers,index):
    visibleMarkers=[]
    for marker in markers:
        if acq.GetPoint(utils.str(marker)).GetResidual(index) !=-1:
            visibleMarkers.append(marker)
    return visibleMarkers




def isAnalogExist(acq,label):
    """
        Check if a point label exists inside an acquisition

        :Parameters:
            - `acq` (btkAcquisition) - a btk acquisition inctance
            - `label` (str) - point label
    """
    #TODO : replace by btkIterate
    i = acq.GetAnalogs().Begin()
    while i != acq.GetAnalogs().End():
        if i.value().GetLabel()==utils.str(label):
            flag= True
            break
        else:
            i.incr()
            flag= False

    if flag:
        return True
    else:
        return False



def smartAppendAnalog(acq,label,values,desc="" ):

    if isAnalogExist(acq,label):
        acq.GetAnalog(utils.str(label)).SetValues(values)
        acq.GetAnalog(utils.str(label)).SetDescription(utils.str(desc))
        #acq.GetAnalog(label).SetType(PointType)

    else:
        newAnalog=btk.btkAnalog(acq.GetAnalogFrameNumber())
        newAnalog.SetValues(values)
        newAnalog.SetLabel(utils.str(label))
        acq.AppendAnalog(newAnalog)


def markerUnitConverter(acq,unitOffset):
    for it in btk.Iterate(acq.GetPoints()):
        if it.GetType() == btk.btkPoint.Marker:
            values = it.GetValues()
            it.SetValues(values*unitOffset)

def constructMarker(acq,label,markers,numpyMethod=np.mean,desc=""):
    nFrames = acq.GetPointFrameNumber()

    x=np.zeros((nFrames, len(markers)))
    y=np.zeros((nFrames, len(markers)))
    z=np.zeros((nFrames, len(markers)))

    i=0
    for marker in markers:
        x[:,i] = acq.GetPoint(utils.str(marker)).GetValues()[:,0]
        y[:,i] = acq.GetPoint(utils.str(marker)).GetValues()[:,1]
        z[:,i] = acq.GetPoint(utils.str(marker)).GetValues()[:,2]
        i+=1

    values = np.zeros((nFrames,3))
    values[:,0] = numpyMethod(x, axis=1)
    values[:,1] = numpyMethod(y, axis=1)
    values[:,2] = numpyMethod(z, axis=1)
    smartAppendPoint(acq,label,values,desc=desc)

def constructEmptyMarker(acq,label,desc=""):
    nFrames = acq.GetPointFrameNumber()
    values = np.zeros((nFrames,3))
    smartAppendPoint(acq,label,values,desc=desc,residuals= np.ones((nFrames,1))*-1.0)


def createZeros(acq, markerLabels):
    for label in markerLabels:
        constructEmptyMarker(acq,label,desc="")


def getNumberOfForcePlate(btkAcq):

    pfe = btk.btkForcePlatformsExtractor()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    pfc.Update()

    return pfc.GetItemNumber()


def getStartEndEvents(btkAcq,context,startLabel="Start", endLabel="End"):
    events= btkAcq.GetEvents()

    start=[]
    end=[]
    for ev in btk.Iterate(events):
        if ev.GetContext()== utils.str(context) and ev.GetLabel()==utils.str(startLabel) :
                start.append(ev.GetFrame())
        if ev.GetContext()==utils.str(context) and ev.GetLabel()==utils.str(endLabel) :
                end.append(ev.GetFrame())
    if start==[] or end==[]:
        return None,None

    if len(start)>1 or len(end)>1:
        raise("[pyCGM2]: You can t have multiple Start and End events" )
    elif end<start:
        raise("[pyCGM2]: wrong order ( start<end)" )
    else:
        return start[0],end[0]


def _getSectionFromMd(md):
    md_sections=list()
    for i in range(0, md.GetChildNumber()):
        md_sections.append(md.GetChild(i).GetLabel())
    return md_sections


def changeSubjectName(btkAcq,subjectName):


    # change subject name in the metadata
    md = btkAcq.GetMetaData()


    if "SUBJECTS" in _getSectionFromMd(md):
        subjectMd =  btkAcq.GetMetaData().FindChild(utils.str("SUBJECTS")).value()
        if "NAMES" in _getSectionFromMd(subjectMd):
            subjectMd.FindChild(utils.str("NAMES")).value().GetInfo().SetValue(0,utils.str(subjectName))

        if "USES_PREFIXES"  not in _getSectionFromMd(subjectMd):
            btk.btkMetaDataCreateChild(subjectMd, utils.str("USES_PREFIXES"), 0)

    if "ANALYSIS" in _getSectionFromMd(md):
        analysisMd =  btkAcq.GetMetaData().FindChild(utils.str("ANALYSIS")).value()
        if "SUBJECTS" in _getSectionFromMd(analysisMd):
            anaSubMdi = analysisMd.FindChild(utils.str("SUBJECTS")).value().GetInfo()
            for i in range(0,anaSubMdi.GetDimension(1) ):
                anaSubMdi.SetValue(i,utils.str(subjectName))

    events = btkAcq.GetEvents()
    for ev in btk.Iterate(events):
        ev.SetSubject(utils.str(subjectName))

    # Do not work
    # eventMd =  btkAcq.GetMetaData().FindChild("EVENT").value()
    # eventMdi = eventMd.FindChild("SUBJECTS").value().GetInfo()
    # for i in range(0,eventMdi.GetDimension(1) ):
    #     eventMdi.SetValue(i,"TEST")

    return btkAcq

def smartGetMetadata(btkAcq,firstLevel,secondLevel):
    md = btkAcq.GetMetaData()
    if secondLevel is not None:
        if utils.str(firstLevel) in _getSectionFromMd(md):
            firstMd =  btkAcq.GetMetaData().FindChild(utils.str(firstLevel)).value()
            if utils.str(secondLevel) in _getSectionFromMd(firstMd):
                return firstMd.FindChild(utils.str(secondLevel)).value().GetInfo().ToString()
    else:
        if utils.str(firstLevel) in _getSectionFromMd(md):
            return md.FindChild(utils.str(firstLevel)).value().GetInfo().ToString()

def smartSetMetadata(btkAcq,firstLevel,secondLevel,index,value):
    md = btkAcq.GetMetaData()
    if utils.str(firstLevel) in _getSectionFromMd(md):
        firstMd =  btkAcq.GetMetaData().FindChild(utils.str(firstLevel)).value()
        if utils.str(secondLevel) in _getSectionFromMd(firstMd):
            return firstMd.FindChild(utils.str(secondLevel)).value().GetInfo().SetValue(index,utils.str(value))

def checkMetadata(btkAcq,firstLevel,secondLevel):
    md = btkAcq.GetMetaData()
    flag =  False
    if secondLevel is not None:
        if utils.str(firstLevel) in _getSectionFromMd(md):
            firstMd =  btkAcq.GetMetaData().FindChild(utils.str(firstLevel)).value()
            if utils.str(secondLevel) in _getSectionFromMd(firstMd):
                flag =  True
    else:
        if utils.str(firstLevel) in _getSectionFromMd(md):
            flag = True

    return flag

def checkForcePlateExist(btkAcq):

    if checkMetadata(btkAcq,"FORCE_PLATFORM","USED"):
        if  smartGetMetadata(btkAcq,"FORCE_PLATFORM","USED")[0] != "0":
            return True
        else:
            return False
    else:
        return False




def NexusGetTrajectory(acq,label):
    values = acq.GetPoint(utils.str(label)).GetValues()
    residuals = acq.GetPoint(utils.str(label)).GetResiduals()[:,0]
    residualsBool = np.asarray(residuals)==0

    return values[:,0],values[:,1],values[:,2],residualsBool


def sortedEvents(acq):
    evs = acq.GetEvents()

    contextLst=[] # recuperation de tous les contextes
    for it in btk.Iterate(evs):
        if it.GetContext() not in contextLst:
            contextLst.append(it.GetContext())


    valueFrame=[] # recuperation de toutes les frames SANS doublons
    for it in  btk.Iterate(evs):
        if it.GetFrame() not in valueFrame:
            valueFrame.append(it.GetFrame())
    valueFrame.sort() # trie

    events =[]
    for frame in valueFrame:
        for it in  btk.Iterate(evs):
            if it.GetFrame()==frame:
                events.append(it)

    newEvents=btk.btkEventCollection()
    for ev in events:
        newEvents.InsertItem(ev)

    acq.ClearEvents()
    acq.SetEvents(newEvents)


#------------------- FROM trials TOOLS------------------------------------------

def buildTrials(dataPath,filenames):

    acqs=[]
    acqFilenames =[]
    for filename in filenames:
        logging.debug( dataPath)
        logging.debug( filename)
        acq = smartReader(dataPath + filename)
        sortedEvents(acq)

        acqs.append(acq)
        acqFilenames.append(filename)

    return acqs, acqFilenames


def isKineticFlag(acq):
    """
        Flag up if correct kinetics available

        :Parameters:
            - `trial` (openma.trial) - an openma trial instance

        :Return:
            - `` (bool) - flag if kinetic available
            - `kineticEvent_times` (lst) - time of maximal Normal reaction Forces for both context
            - `kineticEvent_times_left` (lst) - time of maximal Normal reaction Forces for the Left context
            - `kineticEvent_times_right` (lst) - time of maximal Normal reaction Forces for the Right context
    """


    kineticEvent_frames=[]
    kineticEvent_frames_left=[]
    kineticEvent_frames_right=[]

    events= acq.GetEvents()
    for ev in btk.Iterate(events):
        if ev.GetContext() == "General":
            if ev.GetLabel() in ["Left-FP","Right-FP"]:
                kineticEvent_frames.append(ev.GetFrame())
            if ev.GetLabel() in ["Left-FP"]:
                kineticEvent_frames_left.append(ev.GetFrame())
            if ev.GetLabel() in ["Right-FP"]:
                kineticEvent_frames_right.append(ev.GetFrame())

    if kineticEvent_frames==[]:
        return False,0,0,0
    else:
        return True,kineticEvent_frames,kineticEvent_frames_left,kineticEvent_frames_right

def automaticKineticDetection(dataPath,filenames,acqs=None):
    """
        convenient method for detecting correct kinetic in a filename set

        :Parameters:
            - `dataPath` (str) - folder path
            - `filenames` (list of str) - filename of the different acquisitions
    """
    kineticAcqs=[]
    kineticFilenames=[]

    i=0
    for filename in filenames:
        if filename in kineticFilenames:
            logging.debug("[pyCGM2] : filename %s duplicated in the input list" %(filename))
        else:
            if acqs is None:
                acq = smartReader(dataPath + filename)

            else:
                acq = acqs[i]

            sortedEvents(acq)
            flag_kinetics,times, times_l, times_r = isKineticFlag(acq)

            if flag_kinetics:
                kineticFilenames.append(filename)
                kineticAcqs.append(acq)
    i+=1

    kineticAcqs = None if kineticAcqs ==[] else kineticAcqs
    flag_kinetics = False if kineticAcqs ==[] else True


    return kineticAcqs,kineticFilenames,flag_kinetics

def getForcePlateWrench(btkAcq,fpIndex=None):
    # --- ground reaction force wrench ---
    pfe = btk.btkForcePlatformsExtractor()
    grwf = btk.btkGroundReactionWrenchFilter()
    pfe.SetInput(btkAcq)
    pfc = pfe.GetOutput()
    grwf.SetInput(pfc)
    grwc = grwf.GetOutput()
    grwc.Update()

    if fpIndex is not None:
        return grwc.GetItem(fpIndex-1)
    else:
        return grwc



# -*- coding: utf-8 -*-
import numpy as np
from pyCGM2 import enums


def getNexusSubjectMp(NEXUS,subject, resetFlag=False):

    params = NEXUS.GetSubjectParamNames(subject)

    required_mp={
    'Bodymass'   : NEXUS.GetSubjectParamDetails( subject, "Bodymass")[0] if "Bodymass" in params else 0,#71.0,
    'LeftLegLength' : NEXUS.GetSubjectParamDetails( subject, "LeftLegLength")[0] if "LeftLegLength" in params else 0,#860.0,
    'RightLegLength' : NEXUS.GetSubjectParamDetails( subject, "RightLegLength")[0] if "RightLegLength" in params else 0,#865.0 ,
    'LeftKneeWidth' : NEXUS.GetSubjectParamDetails( subject, "LeftKneeWidth")[0]if "LeftKneeWidth" in params else 0,#102.0,
    'RightKneeWidth' : NEXUS.GetSubjectParamDetails( subject, "RightKneeWidth")[0] if "RightKneeWidth" in params else 0,#103.4,
    'LeftAnkleWidth' : NEXUS.GetSubjectParamDetails( subject, "LeftAnkleWidth")[0] if "LeftAnkleWidth" in params else 0,#75.3,
    'RightAnkleWidth' : NEXUS.GetSubjectParamDetails( subject, "RightAnkleWidth")[0] if "RightAnkleWidth" in params else 0,#72.9,
    'LeftSoleDelta' : NEXUS.GetSubjectParamDetails( subject, "LeftSoleDelta")[0] if "LeftSoleDelta" in params else 0,#75.3,
    'RightSoleDelta' : NEXUS.GetSubjectParamDetails( subject, "RightSoleDelta")[0] if "RightSoleDelta" in params else 0,#72.9,
    'LeftShoulderOffset' : NEXUS.GetSubjectParamDetails( subject, "LeftShoulderOffset")[0] if "LeftShoulderOffset" in params else 0,#72.9,
    'RightShoulderOffset' : NEXUS.GetSubjectParamDetails( subject, "RightShoulderOffset")[0] if "RightShoulderOffset" in params else 0,#72.9,
    'LeftElbowWidth' : NEXUS.GetSubjectParamDetails( subject, "LeftElbowWidth")[0] if "LeftElbowWidth" in params else 0,#72.9,
    'LeftWristWidth' : NEXUS.GetSubjectParamDetails( subject, "LeftWristWidth")[0] if "LeftWristWidth" in params else 0,#72.9,
    'LeftHandThickness' : NEXUS.GetSubjectParamDetails( subject, "LeftHandThickness")[0] if "LeftHandThickness" in params else 0,#72.9,
    'RightElbowWidth' : NEXUS.GetSubjectParamDetails( subject, "RightElbowWidth")[0] if "RightElbowWidth" in params else 0,#72.9,
    'RightWristWidth' : NEXUS.GetSubjectParamDetails( subject, "RightWristWidth")[0] if "RightWristWidth" in params else 0,#72.9,
    'RightHandThickness' : NEXUS.GetSubjectParamDetails( subject, "RightHandThickness")[0] if "RightHandThickness" in params else 0,#72.9,
    }

    optional_mp={
    'InterAsisDistance'   : NEXUS.GetSubjectParamDetails( subject, "InterAsisDistance")[0] if "InterAsisDistance" in params else 0,#0,
    'LeftAsisTrocanterDistance' : NEXUS.GetSubjectParamDetails( subject, "LeftAsisTrocanterDistance")[0] if "LeftAsisTrocanterDistance" in params else 0,#0,
    'RightAsisTrocanterDistance' : NEXUS.GetSubjectParamDetails( subject, "RightAsisTrocanterDistance")[0] if "RightAsisTrocanterDistance" in params else 0,#0,
    }


    if resetFlag:
        optional_mp.update({
        'LeftTibialTorsion' : 0 ,
        'LeftThighRotation' : 0,
        'LeftShankRotation' : 0,
        'RightTibialTorsion' :0 ,
        'RightThighRotation' : 0,
        'RightShankRotation' : 0
        })

    else:
        optional_mp.update({
        'LeftTibialTorsion' : NEXUS.GetSubjectParamDetails( subject, "LeftTibialTorsion")[0] if "LeftTibialTorsion" in params else 0,#0 ,
        'LeftThighRotation' : NEXUS.GetSubjectParamDetails( subject, "LeftThighRotation")[0] if "LeftThighRotation" in params else 0,#0,
        'LeftShankRotation' : NEXUS.GetSubjectParamDetails( subject, "LeftShankRotation")[0] if "LeftShankRotation" in params else 0,#0,
        'RightTibialTorsion' : NEXUS.GetSubjectParamDetails( subject, "RightTibialTorsion")[0] if "RightTibialTorsion" in params else 0,#0 ,
        'RightThighRotation' : NEXUS.GetSubjectParamDetails( subject, "RightThighRotation")[0] if "RightThighRotation" in params else 0,#0,
        'RightShankRotation' : NEXUS.GetSubjectParamDetails( subject, "RightShankRotation")[0] if "RightShankRotation" in params else 0,#0,
        })
    return required_mp,optional_mp

def updateNexusSubjectMp(NEXUS,model,subjectName):

    NEXUS.SetSubjectParam( subjectName, "InterAsisDistance",model.mp_computed["InterAsisDistance"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftAsisTrocanterDistance",model.mp_computed["LeftAsisTrocanterDistance"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftThighRotation",model.mp_computed["LeftThighRotationOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftShankRotation",model.mp_computed["LeftShankRotationOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftTibialTorsion",model.mp_computed["LeftTibialTorsionOffset"],True)


    NEXUS.SetSubjectParam( subjectName, "RightAsisTrocanterDistance",model.mp_computed["RightAsisTrocanterDistance"],True)
    NEXUS.SetSubjectParam( subjectName, "RightThighRotation",model.mp_computed["RightThighRotationOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "RightShankRotation",model.mp_computed["RightShankRotationOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "RightTibialTorsion",model.mp_computed["RightTibialTorsionOffset"],True)


    NEXUS.SetSubjectParam( subjectName, "LeftStaticPlantFlex",model.mp_computed["LeftStaticPlantFlexOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftStaticRotOff",model.mp_computed["LeftStaticRotOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "LeftAnkleAbAdd",model.mp_computed["LeftAnkleAbAddOffset"],True)

    NEXUS.SetSubjectParam( subjectName, "RightStaticPlantFlex",model.mp_computed["RightStaticPlantFlexOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "RightStaticRotOff",model.mp_computed["RightStaticRotOffset"],True)
    NEXUS.SetSubjectParam( subjectName, "RightAnkleAbAdd",model.mp_computed["RightAnkleAbAddOffset"],True)



# -*- coding: utf-8 -*-
from pyCGM2.Utils.utils import toBool
from datetime import datetime
import logging


def getFilename(measurement):
    return measurement.attrs["Filename"].replace("qtm","c3d")


def getForcePlateAssigment(measurement):
    mea = measurement

    mfpa = mea.find("Forceplate1").text[0] + mea.find("Forceplate2").text[0]+ \
    mea.find("Forceplate3").text[0] + mea.find("Forceplate4").text[0] + \
    mea.find("Forceplate5").text[0]

    return mfpa


def isType(measurement,type):
    return measurement.attrs["Type"] == type



def findStatic(soup):
    qtmMeasurements = soup.find_all("Measurement")
    static=list()
    for measurement in qtmMeasurements:
        if "Static" in measurement.attrs["Type"] and toBool(measurement.Used.text):
            static.append(measurement)
        if len(static)>1:
            raise Exception("You can t have 2 activated static c3d within your session")
    return static[0]

def findDynamic(soup):
    qtmMeasurements = soup.find_all("Measurement")

    measurements=list()
    for measurement in qtmMeasurements:
        if "Gait" in measurement.attrs["Type"] and toBool(measurement.Used.text):
            measurements.append(measurement)

    return measurements


def detectMeasurementType(soup):
    measurements = soup.find_all("Measurement")

    types = list()
    for measurement in measurements:
        if "Static" not in measurement.attrs["Type"] and toBool(measurement.Used.text):
            if measurement.attrs["Type"] not in types:
                types.append(measurement.attrs["Type"])

    return types



def SubjectMp(soup):

    bodymass = float(soup.Subject.Weight.text)
    if bodymass == 0.0:
        logging.error("[pyCGM2] Null Bodymass detected - Kinetics will be unnormalized")
        bodymass = 1.0

    required_mp={
    'Bodymass'   : bodymass,
    'Height'   : float(soup.Subject.Height.text),
    'LeftLegLength' : float(soup.Subject.Leg_length_left.text)*1000.0,
    'RightLegLength' : float(soup.Subject.Leg_length_right.text)*1000.0,#865.0 ,
    'LeftKneeWidth' : float(soup.Subject.Knee_width_left.text)*1000.0,#102.0,
    'RightKneeWidth' : float(soup.Subject.Knee_width_right.text)*1000.0,#103.4,
    'LeftAnkleWidth' : float(soup.Subject.Ankle_width_left.text)*1000.0,#75.3,
    'RightAnkleWidth' : float(soup.Subject.Ankle_width_right.text)*1000.0,#72.9,
    'LeftSoleDelta' : float(soup.Subject.Sole_delta_left.text)*1000.0,#75.3,
    'RightSoleDelta' : float(soup.Subject.Sole_delta_right.text)*1000.0,#72.9,
    'LeftShoulderOffset' : float(soup.Subject.Shoulder_offset_left.text)*1000.0,#72.9,
    'RightShoulderOffset' : float(soup.Subject.Shoulder_offset_right.text)*1000.0,#72.9,
    'LeftElbowWidth' : float(soup.Subject.Elbow_width_left.text)*1000.0,#72.9,
    'LeftWristWidth' : float(soup.Subject.Wrist_width_left.text)*1000.0,#72.9,
    'LeftHandThickness' : float(soup.Subject.Hand_thickness_left.text)*1000.0,#72.9,
    'RightElbowWidth' : float(soup.Subject.Elbow_width_right.text)*1000.0,#72.9,
    'RightWristWidth' : float(soup.Subject.Wrist_width_right.text)*1000.0,#72.9,
    'RightHandThickness' : float(soup.Subject.Hand_thickness_right.text)*1000.0#72.9,
    }

    optional_mp={
        'InterAsisDistance'   : 0,
        'LeftAsisTrocanterDistance' : 0,
        'LeftTibialTorsion' : 0 ,
        'LeftThighRotation' : 0,
        'LeftShankRotation' : 0,
        'RightAsisTrocanterDistance' : 0,
        'RightTibialTorsion' :0 ,
        'RightThighRotation' : 0,
        'RightShankRotation' : 0
        }

    return required_mp,optional_mp


def get_modelled_trials(session_xml, measurement_type):
    modelled_trials = []
    dynamicMeasurements = findDynamic(session_xml)
    for dynamicMeasurement in dynamicMeasurements:
        if isType(dynamicMeasurement, measurement_type):
            filename = getFilename(dynamicMeasurement)
            modelled_trials.append(filename)
    return modelled_trials


def get_creation_date(session_xml):
    date_str = session_xml.Subject.Session.Creation_date.text
    year, month, day = date_str.split("-")
    time_str = session_xml.Subject.Session.Creation_time.text
    hour, minute, second = time_str.split(":")
    datetime_obj = datetime(
        int(year), int(month), int(day),
        int(hour), int(minute), int(second))
    return datetime_obj



# -*- coding: utf-8 -*-
import logging
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from pyCGM2.Lib import analysis
from pyCGM2.Lib import plot
from pyCGM2 import enums

def pdfGaitReport(DATA_PATH,model,modelledTrials, normativeDataset,pointSuffix, title = "gait report"):

    analysisInstance = analysis.makeAnalysis(
        DATA_PATH,modelledTrials,
        subjectInfo=None,
        experimentalInfo=None,
        modelInfo=None,
        pointLabelSuffix=None)

    with PdfPages(DATA_PATH + title+".pdf") as pdf:
        # spatiotemporal
        plot.plot_spatioTemporal(DATA_PATH,analysisInstance,
            exportPdf=False,
            outputName=title,
            show=None,
            title=title)
        pdf.savefig()

        #Kinematics
        if model.m_bodypart in [enums.BodyPart.LowerLimb,enums.BodyPart.LowerLimbTrunk, enums.BodyPart.FullBody]:
            plot.plot_DescriptiveKinematic(DATA_PATH,analysisInstance,"LowerLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

            plot.plot_ConsistencyKinematic(DATA_PATH,analysisInstance,"LowerLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

        if model.m_bodypart in [enums.BodyPart.LowerLimbTrunk, enums.BodyPart.FullBody]:
            plot.plot_DescriptiveKinematic(DATA_PATH,analysisInstance,"Trunk",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

            plot.plot_ConsistencyKinematic(DATA_PATH,analysisInstance,"Trunk",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

        if model.m_bodypart in [enums.BodyPart.UpperLimb, enums.BodyPart.FullBody]:
            plot.plot_DescriptiveKinematic(DATA_PATH,analysisInstance,"UpperLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

            plot.plot_ConsistencyKinematic(DATA_PATH,analysisInstance,"UpperLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()


        #Kinetics
        if model.m_bodypart in [enums.BodyPart.LowerLimb,enums.BodyPart.LowerLimbTrunk, enums.BodyPart.FullBody]:
            plot.plot_DescriptiveKinetic(DATA_PATH,analysisInstance,"LowerLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

            plot.plot_ConsistencyKinetic(DATA_PATH,analysisInstance,"LowerLimb",
                normativeDataset,
                exportPdf=False,
                outputName=title,
                pointLabelSuffix=pointSuffix,
                show=False,
                title=title)
            pdf.savefig()

        #MAP
        plot.plot_MAP(DATA_PATH,analysisInstance,
            normativeDataset,
            exportPdf=False,
            outputName=title,pointLabelSuffix=pointSuffix,
            show=False,
            title=title)
        pdf.savefig()

        plt.show(False)
        logging.info("----- Gait Processing -----> DONE")



# -*- coding: utf-8 -*-
#import ipdb
import logging
import matplotlib.pyplot as plt
import argparse


# pyCGM2 settings
import pyCGM2

# pyCGM2 libraries
from pyCGM2.Tools import btkTools
from pyCGM2 import enums

from pyCGM2.Model import modelFilters, modelDecorator,bodySegmentParameters
from pyCGM2.Model.CGM2 import cgm
from pyCGM2.Model.CGM2 import decorators
from pyCGM2.ForcePlates import forceplates
from pyCGM2.Processing import progressionFrame
from pyCGM2.Signal import signal_processing


def calibrate(DATA_PATH,calibrateFilenameLabelled,translators,
              required_mp,optional_mp,
              leftFlatFoot,rightFlatFoot,headFlat,markerDiameter,
              pointSuffix,**kwargs):
    """
    Calibration of the CGM1

    :param DATA_PATH [str]: path to your data
    :param calibrateFilenameLabelled [str]: c3d file
    :param translators [dict]:  translators to apply
    :param required_mp [dict]: required anthropometric data
    :param optional_mp [dict]: optional anthropometric data (ex: LThighOffset,...)
    :param leftFlatFoot [bool]: enable of the flat foot option for the left foot
    :param rightFlatFoot [bool]: enable of the flat foot option for the right foot
    :param headFlat [bool]: enable of the head flat  option
    :param markerDiameter [double]: marker diameter (mm)
    :param pointSuffix [str]: suffix to add to model outputs

    """

    # --------------------------ACQUISITION ------------------------------------

    if "forceBtkAcq" in kwargs.keys():
        acqStatic = kwargs["forceBtkAcq"]
    else:
    # ---btk acquisition---
        acqStatic = btkTools.smartReader((DATA_PATH+calibrateFilenameLabelled))

    btkTools.checkMultipleSubject(acqStatic)
    if btkTools.isPointExist(acqStatic,"SACR"):
        translators["LPSI"] = "SACR"
        translators["RPSI"] = "SACR"
        logging.info("[pyCGM2] Sacrum marker detected")

    acqStatic =  btkTools.applyTranslators(acqStatic,translators)


    # ---check marker set used----
    dcm= cgm.CGM.detectCalibrationMethods(acqStatic)

    # ---definition---
    model=cgm.CGM1()
    model.configure(acq=acqStatic,detectedCalibrationMethods=dcm)
    model.addAnthropoInputParameters(required_mp,optional=optional_mp)

    # --store calibration parameters--
    model.setStaticFilename(calibrateFilenameLabelled)
    model.setCalibrationProperty("leftFlatFoot",leftFlatFoot)
    model.setCalibrationProperty("rightFlatFoot",rightFlatFoot)
    model.setCalibrationProperty("headFlat",headFlat)
    model.setCalibrationProperty("markerDiameter",markerDiameter)


    # --------------------------STATIC CALBRATION--------------------------
    scp=modelFilters.StaticCalibrationProcedure(model) # load calibration procedure

    # ---initial calibration filter----
    modelFilters.ModelCalibrationFilter(scp,acqStatic,model,
                                        leftFlatFoot = leftFlatFoot,
                                        rightFlatFoot = rightFlatFoot,
                                        markerDiameter = markerDiameter,
                                        headFlat= headFlat,
                                        viconCGM1compatible=True
                                        ).compute()
    # ---- Decorators -----
    decorators.applyBasicDecorators(dcm, model,acqStatic,optional_mp,markerDiameter,cgm1only=True)
    pigStaticMarkers = cgm.CGM.get_markerLabelForPiGStatic(dcm)

    # ----Final Calibration filter if model previously decorated -----
    if model.decoratedModel:
        # initial static filter
        modelFilters.ModelCalibrationFilter(scp,acqStatic,model,
                           leftFlatFoot = leftFlatFoot, rightFlatFoot = rightFlatFoot,
                           headFlat= headFlat,
                           markerDiameter=markerDiameter,
                           viconCGM1compatible=True).compute()


    # ----------------------CGM MODELLING----------------------------------
    # ----motion filter----
    # notice : viconCGM1compatible option duplicate error on Construction of the foot coordinate system

    modMotion=modelFilters.ModelMotionFilter(scp,acqStatic,model,enums.motionMethod.Determinist,
                                                  markerDiameter=markerDiameter,
                                                  viconCGM1compatible=False,
                                                  pigStatic=True,
                                                  useLeftKJCmarker=pigStaticMarkers[0], useLeftAJCmarker=pigStaticMarkers[1],
                                                  useRightKJCmarker=pigStaticMarkers[2], useRightAJCmarker=pigStaticMarkers[3],
                                                  leftFlatFoot = leftFlatFoot,
                                                  rightFlatFoot = rightFlatFoot)


    modMotion.compute()


    if "displayCoordinateSystem" in kwargs.keys() and kwargs["displayCoordinateSystem"]:
        csp = modelFilters.ModelCoordinateSystemProcedure(model)
        csdf = modelFilters.CoordinateSystemDisplayFilter(csp,model,acqStatic)
        csdf.setStatic(False)
        csdf.display()

    if "noKinematicsCalculation" in kwargs.keys() and kwargs["noKinematicsCalculation"]:
        logging.warning("[pyCGM2] No Kinematic calculation done for the static file")
        return model, acqStatic
    else:
        #---- Joint kinematics----
        # relative angles
        modelFilters.ModelJCSFilter(model,acqStatic).compute(description="vectoriel", pointLabelSuffix=pointSuffix)


        # detection of traveling axis + absolute angle
        if model.m_bodypart != enums.BodyPart.UpperLimb:
            pfp = progressionFrame.PelvisProgressionFrameProcedure()
        else:
            pfp = progressionFrame.ThoraxProgressionFrameProcedure()

        pff = progressionFrame.ProgressionFrameFilter(acqStatic,pfp)
        pff.compute()
        globalFrame = pff.outputs["globalFrame"]
        forwardProgression = pff.outputs["forwardProgression"]

        if model.m_bodypart != enums.BodyPart.UpperLimb:
                modelFilters.ModelAbsoluteAnglesFilter(model,acqStatic,
                                                       segmentLabels=["Left Foot","Right Foot","Pelvis"],
                                                        angleLabels=["LFootProgress", "RFootProgress","Pelvis"],
                                                        eulerSequences=["TOR","TOR", "TOR"],
                                                        globalFrameOrientation = globalFrame,
                                                        forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

        if model.m_bodypart == enums.BodyPart.LowerLimbTrunk:
                modelFilters.ModelAbsoluteAnglesFilter(model,acqStatic,
                                              segmentLabels=["Thorax"],
                                              angleLabels=["Thorax"],
                                              eulerSequences=["YXZ"],
                                              globalFrameOrientation = globalFrame,
                                              forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

        if model.m_bodypart == enums.BodyPart.UpperLimb or model.m_bodypart == enums.BodyPart.FullBody:

                modelFilters.ModelAbsoluteAnglesFilter(model,acqStatic,
                                              segmentLabels=["Thorax","Head"],
                                              angleLabels=["Thorax", "Head"],
                                              eulerSequences=["YXZ","TOR"],
                                              globalFrameOrientation = globalFrame,
                                              forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)
        # BSP model
        bspModel = bodySegmentParameters.Bsp(model)
        bspModel.compute()

        if  model.m_bodypart == enums.BodyPart.FullBody:
            modelFilters.CentreOfMassFilter(model,acqStatic).compute(pointLabelSuffix=pointSuffix)

        return model, acqStatic


def fitting(model,DATA_PATH, reconstructFilenameLabelled,
    translators,
    markerDiameter,
    pointSuffix,
    mfpa,
    momentProjection,**kwargs):

    """
    Fitting of the CGM1

    :param model [str]: pyCGM2 model previously calibrated
    :param DATA_PATH [str]: path to your data
    :param reconstructFilenameLabelled [string list]: c3d files
    :param translators [dict]:  translators to apply
    :param mfpa [str]: manual force plate assignement
    :param markerDiameter [double]: marker diameter (mm)
    :param pointSuffix [str]: suffix to add to model outputs
    :param momentProjection [str]: Coordinate system in which joint moment is expressed

    """

    # --------------------------ACQUISITION ------------------------------------

    if "forceBtkAcq" in kwargs.keys():
        acqGait = kwargs["forceBtkAcq"]
    else:
        # --- btk acquisition ----
        acqGait = btkTools.smartReader((DATA_PATH + reconstructFilenameLabelled))

    btkTools.checkMultipleSubject(acqGait)
    if btkTools.isPointExist(acqGait,"SACR"):
        translators["LPSI"] = "SACR"
        translators["RPSI"] = "SACR"
        logging.info("[pyCGM2] Sacrum marker detected")

    acqGait =  btkTools.applyTranslators(acqGait,translators)

    trackingMarkers = model.getTrackingMarkers(acqGait)
    validFrames,vff,vlf = btkTools.findValidFrames(acqGait,trackingMarkers)

    # filtering
    # -----------------------
    if "fc_lowPass_marker" in kwargs.keys() and kwargs["fc_lowPass_marker"]!=0 :
        fc = kwargs["fc_lowPass_marker"]
        order = 4
        if "order_lowPass_marker" in kwargs.keys():
            order = kwargs["order_lowPass_marker"]
        signal_processing.markerFiltering(acqGait,trackingMarkers,order=order, fc =fc)

    if "fc_lowPass_forcePlate" in kwargs.keys() and kwargs["fc_lowPass_forcePlate"]!=0 :
        fc = kwargs["fc_lowPass_forcePlate"]
        order = 4
        if "order_lowPass_forcePlate" in kwargs.keys():
            order = kwargs["order_lowPass_forcePlate"]
        signal_processing.forcePlateFiltering(acqGait,order=order, fc =fc)

    scp=modelFilters.StaticCalibrationProcedure(model) # procedure
    # ---Motion filter----
    modMotion=modelFilters.ModelMotionFilter(scp,acqGait,model,enums.motionMethod.Determinist,
                                              markerDiameter=markerDiameter,
                                              viconCGM1compatible=True)

    modMotion.compute()

    if "displayCoordinateSystem" in kwargs.keys() and kwargs["displayCoordinateSystem"]:
        csp = modelFilters.ModelCoordinateSystemProcedure(model)
        csdf = modelFilters.CoordinateSystemDisplayFilter(csp,model,acqGait)
        csdf.setStatic(False)
        csdf.display()

    #---- Joint kinematics----
    # relative angles
    modelFilters.ModelJCSFilter(model,acqGait).compute(description="vectoriel", pointLabelSuffix=pointSuffix)

    # detection of traveling axis + absolute angle
    if model.m_bodypart != enums.BodyPart.UpperLimb:
        pfp = progressionFrame.PelvisProgressionFrameProcedure()
    else:
        pfp = progressionFrame.ThoraxProgressionFrameProcedure()

    pff = progressionFrame.ProgressionFrameFilter(acqGait,pfp)
    pff.compute()
    globalFrame = pff.outputs["globalFrame"]
    forwardProgression = pff.outputs["forwardProgression"]

    if model.m_bodypart != enums.BodyPart.UpperLimb:
            modelFilters.ModelAbsoluteAnglesFilter(model,acqGait,
                                                   segmentLabels=["Left Foot","Right Foot","Pelvis"],
                                                    angleLabels=["LFootProgress", "RFootProgress","Pelvis"],
                                                    eulerSequences=["TOR","TOR", "TOR"],
                                                    globalFrameOrientation = globalFrame,
                                                    forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    if model.m_bodypart == enums.BodyPart.LowerLimbTrunk:
            modelFilters.ModelAbsoluteAnglesFilter(model,acqGait,
                                          segmentLabels=["Thorax"],
                                          angleLabels=["Thorax"],
                                          eulerSequences=["YXZ"],
                                          globalFrameOrientation = globalFrame,
                                          forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    if model.m_bodypart == enums.BodyPart.UpperLimb or model.m_bodypart == enums.BodyPart.FullBody:

            modelFilters.ModelAbsoluteAnglesFilter(model,acqGait,
                                          segmentLabels=["Thorax","Head"],
                                          angleLabels=["Thorax", "Head"],
                                          eulerSequences=["YXZ","TOR"],
                                          globalFrameOrientation = globalFrame,
                                          forwardProgression = forwardProgression).compute(pointLabelSuffix=pointSuffix)

    #---- Body segment parameters----
    bspModel = bodySegmentParameters.Bsp(model)
    bspModel.compute()

    #---- CentreOfMass----
    if  model.m_bodypart == enums.BodyPart.FullBody:
        modelFilters.CentreOfMassFilter(model,acqGait).compute(pointLabelSuffix=pointSuffix)

    if btkTools.checkForcePlateExist(acqGait):
        # Inverse dynamics
        if model.m_bodypart != enums.BodyPart.UpperLimb:
            # --- force plate handling----
            # find foot  in contact
            mappedForcePlate = forceplates.matchingFootSideOnForceplate(acqGait,mfpa=mfpa)
            forceplates.addForcePlateGeneralEvents(acqGait,mappedForcePlate)
            logging.warning("Manual Force plate assignment : %s" %mappedForcePlate)


            # assembly foot and force plate
            modelFilters.ForcePlateAssemblyFilter(model,acqGait,mappedForcePlate,
                                     leftSegmentLabel="Left Foot",
                                     rightSegmentLabel="Right Foot").compute(pointLabelSuffix=pointSuffix)


            #---- Joint kinetics----
            if type(momentProjection) == str:
                momentProjection = enums.enumFromtext(momentProjection,enums.MomentProjection)
            idp = modelFilters.CGMLowerlimbInverseDynamicProcedure()
            modelFilters.InverseDynamicFilter(model,
                                 acqGait,
                                 procedure = idp,
                                 projection = momentProjection,
                                 viconCGM1compatible=True,
                                 globalFrameOrientation = globalFrame,
                                 forwardProgression = forwardProgression
                                 ).compute(pointLabelSuffix=pointSuffix)


            #---- Joint energetics----
            modelFilters.JointPowerFilter(model,acqGait).compute(pointLabelSuffix=pointSuffix)

    #---- zero unvalid frames ---
    btkTools.applyValidFramesOnOutput(acqGait,validFrames)

    return acqGait



# -*- coding: utf-8 -*-
#import ipdb
import numpy as np
import pandas as pd
import logging


# pyCGM2
import pyCGM2.Processing.cycle as CGM2cycle
from pyCGM2.Tools import exportTools


class AnalysisStructure:
    data = dict()
    pst = dict()
    optionalData = dict()



# ---- PATTERN BUILDER ------

# --- OBJECT TO BUILD-----
class Analysis():
    """
       Object built from AnalysisFilter.build().

       Analysis work as **class-container**. Its attribute members return descriptive statistics

       Attributes :

          - `stpStats` (dict)  - descritive statictics of stapiotemporal parameters
          - `kinematicStats` (AnalysisStructure)  - descritive statictics of kinematics data.
          - `kineticStats` (AnalysisStructure)  - descritive statictics of kinetics data.
          - `emgStats` (AnalysisStructure)  - descritive statictics of emg data.

       .. note:

           Notice kinematicStats, kineticStats and emgStats are `AnalysisStructure object`. This object implies two dictionnary as sublevels.

             - `data` collect descriptive statistics of either kinematics, kinetics or emg.
             - `pst` returns the spatiotemporal parameters of all cycles involved in either kinematics, kinetics or emg processing.


    """



    def __init__(self):

        self.stpStats=dict()
        self.kinematicStats = AnalysisStructure()
        self.kineticStats=AnalysisStructure()
        self.emgStats=AnalysisStructure()
        self.gps= None
        self.gvs = None
        self.coactivations=dict()
        self.subjectInfo=None
        self.experimentalInfo=None
        self.modelInfo=None

    def setStp(self,inDict):
        self.stpStats = inDict

    def setKinematic(self,data, pst = dict()):
        self.kinematicStats.data = data
        self.kinematicStats.pst = pst

    def setKinetic(self,data, pst = dict(), optionalData=dict()):
        self.kineticStats.data = data
        self.kineticStats.pst = pst
        self.kineticStats.optionalData = optionalData


    def setEmg(self,data, pst = dict()):
        self.emgStats.data = data
        self.emgStats.pst = pst

    def setGps(self,GpsStatsOverall,GpsStatsContext):
        self.gps = dict()
        self.gps["Overall"] = GpsStatsOverall
        self.gps["Context"] = GpsStatsContext

    def setGvs(self,gvsStats):
        self.gvs = gvsStats

    def setSubjectInfo(self,subjectDict):
        self.subjectInfo = subjectDict

    def setExperimentalInfo(self,experDict):
        self.experimentalInfo = experDict

    def setModelInfo(self,modeltDict):
        self.modelInfo = modeltDict

    def getKinematicCycleNumbers(self):
        for label,context in self.kinematicStats.data.keys():
            if context == "Left":
                n_leftCycles = self.kinematicStats.data[label, context]["values"].__len__()
                break

        for label,context in self.kinematicStats.data.keys():
            if context == "Right":
                n_rightCycles = self.kinematicStats.data[label, context]["values"].__len__()
                break
        return n_leftCycles, n_rightCycles

    def setCoactivation(self, labelEmg1,labelEmg2,context,res):
        self.coactivations[labelEmg1,labelEmg2,context]=res

# --- BUILDERS-----
class AbstractBuilder(object):
    def __init__(self,cycles=None):
        self.m_cycles =cycles

    def computeSpatioTemporel(self):
        pass

    def computeKinematics(self):
        pass

    def computeKinetics(self,momentContribution = False):
        pass

    def computeEmgEnvelopes(self):
        pass






class AnalysisBuilder(AbstractBuilder):
    """
        **Description** :
    """

    def __init__(self,cycles,
                 kinematicLabelsDict=None ,
                 kineticLabelsDict =None,
                 pointlabelSuffix = None,
                 emgLabelList = None,
                 modelInfos = None, subjectInfos = None, experimentalInfos = None, emgs = None):

        """
            :Parameters:
                 - `cycles` (pyCGM2.Processing.cycle.Cycles) - Cycles instance built from CycleFilter
                 - `kinematicLabelsDict` (dict) - dictionnary with two items (Left and Right) grouping kinematic output label
                 - `kineticLabelsDict` (dict) - dictionnary with two items (Left and Right) grouping kinetic output label
                 - `pointlabelSuffix` (dict) - suffix ending kinematicLabels and kineticLabels dictionnaries
                 - `emgLabelList` (list of str) - labels of used emg
                 - `subjectInfos` (dict) - information about the subject
                 - `modelInfos` (dict) - information about the model
                 - `experimentalInfos` (dict) - information about the experimental conditions
                 -  .. attention:: `emgs` (pyCGM2emg) - object in progress


            .. note::

                modelInfos,experimentalInfos, subjectInfos are convenient dictionnaries in which you can store different sort of information






        """


        super(AnalysisBuilder, self).__init__(cycles=cycles)

        self.m_kinematicLabelsDict = kinematicLabelsDict
        self.m_kineticLabelsDict = kineticLabelsDict
        self.m_pointlabelSuffix = pointlabelSuffix
        self.m_emgLabelList = emgLabelList
        self.m_emgs = emgs


    def computeSpatioTemporel(self):
        pass

    def computeKinematics(self):
        """ compute descriptive of kinematics parameters

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of kinematics parameters
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching  kinematics parameters

        """

        out={}
        outPst={}

        logging.info("--kinematic computation--")
        if self.m_cycles.kinematicCycles is not None:
            if "Left" in self.m_kinematicLabelsDict.keys():
                for label in self.m_kinematicLabelsDict["Left"]:
                    labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                    out[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kinematicCycles,labelPlus,"Left")

                logging.info("left kinematic computation---> done")
            else:
                logging.warning("No left Kinematic computation")

            if "Right" in self.m_kinematicLabelsDict.keys():
                for label in self.m_kinematicLabelsDict["Right"]:
                    labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                    out[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kinematicCycles,labelPlus,"Right")

                logging.info("right kinematic computation---> done")
            else:
                logging.warning("No right Kinematic computation")

        else:
            logging.warning("No Kinematic computation")

        return out,outPst


    def computeKinetics(self):
        """ compute descriptive of kinetics parameters

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of kinetics parameters
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching  kinetics parameters

        """


        out={}
        outPst={}
        outOptional={}

        logging.info("--kinetic computation--")
        if self.m_cycles.kineticCycles is not None:

           found_context = list()
           for cycle in self.m_cycles.kineticCycles:
               found_context.append(cycle.context)


           if "Left" in self.m_kineticLabelsDict.keys():
               if "Left" in found_context:
                   for label in self.m_kineticLabelsDict["Left"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       out[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Left")
                   for label in self.m_kinematicLabelsDict["Left"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       outOptional[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Left")
                   logging.info("left kinetic computation---> done")
               else:
                   logging.warning("No left Kinetic computation")

           if "Right" in self.m_kineticLabelsDict.keys():
               if  "Right" in found_context:
                   for label in self.m_kineticLabelsDict["Right"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       out[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Right")

                   for label in self.m_kinematicLabelsDict["Right"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       outOptional[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Right")

                   logging.info("right kinetic computation---> done")
               else:
                   logging.warning("No right Kinetic computation")


        else:
            logging.warning("No Kinetic computation")

        return out,outPst,outOptional

    def computeEmgEnvelopes(self):

        """
            Compute descriptive of emg values

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of emg envelopes
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching emg envelopes
        """
        out={}
        outPst={}

        logging.info("--emg computation--")
        if self.m_cycles.emgCycles is not None:
            for rawLabel in self.m_emgLabelList:
                out[rawLabel,"Left"]=CGM2cycle.analog_descriptiveStats(self.m_cycles.emgCycles,rawLabel,"Left")
                out[rawLabel,"Right"]=CGM2cycle.analog_descriptiveStats(self.m_cycles.emgCycles,rawLabel,"Right")

        else:
            logging.warning("No emg computation")

        return out,outPst


class GaitAnalysisBuilder(AbstractBuilder):
    """
        **Description** : builder of a common clinical gait analysis
    """



    def __init__(self,cycles,
                 kinematicLabelsDict=None ,
                 kineticLabelsDict =None,
                 pointlabelSuffix = None,
                 emgLabelList = None,
                 modelInfos = None, subjectInfos = None, experimentalInfos = None, emgs = None):

        """
            :Parameters:
                 - `cycles` (pyCGM2.Processing.cycle.Cycles) - Cycles instance built from CycleFilter
                 - `kinematicLabelsDict` (dict) - dictionnary with two items (Left and Right) grouping kinematic output label
                 - `kineticLabelsDict` (dict) - dictionnary with two items (Left and Right) grouping kinetic output label
                 - `pointlabelSuffix` (dict) - suffix ending kinematicLabels and kineticLabels dictionnaries
                 - `emgLabelList` (list of str) - labels of used emg
                 - `subjectInfos` (dict) - information about the subject
                 - `modelInfos` (dict) - information about the model
                 - `experimentalInfos` (dict) - information about the experimental conditions
                 -  .. attention:: `emgs` (pyCGM2emg) - object in progress


            .. note::

                modelInfos,experimentalInfos, subjectInfos are convenient dictionnaries in which you can store different sort of information






        """


        super(GaitAnalysisBuilder, self).__init__(cycles=cycles)

        self.m_kinematicLabelsDict = kinematicLabelsDict
        self.m_kineticLabelsDict = kineticLabelsDict
        self.m_pointlabelSuffix = pointlabelSuffix
        self.m_emgLabelList = emgLabelList


    def computeSpatioTemporel(self):

        """
            **Description:** compute descriptive of spatio-temporal parameters

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of spatio-temporal parameters

        """
        out={}

        logging.info("--stp computation--")
        if self.m_cycles.spatioTemporalCycles is not None :

            enableLeftComputation = len ([cycle for cycle in self.m_cycles.spatioTemporalCycles if cycle.enableFlag and cycle.context=="Left"])
            enableRightComputation = len ([cycle for cycle in self.m_cycles.spatioTemporalCycles if cycle.enableFlag and cycle.context=="Right"])

            for label in CGM2cycle.GaitCycle.STP_LABELS:
                if enableLeftComputation:
                    out[label,"Left"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.spatioTemporalCycles,label,"Left")

                if enableRightComputation:
                    out[label,"Right"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.spatioTemporalCycles,label,"Right")
            if enableLeftComputation:
                logging.info("left stp computation---> done")
            if enableRightComputation:
                logging.info("right stp computation---> done")
        else:
            logging.warning("No spatioTemporal computation")

        return out

    def computeKinematics(self):
        """ compute descriptive of kinematics parameters

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of kinematics parameters
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching  kinematics parameters

        """

        out={}
        outPst={}

        logging.info("--kinematic computation--")
        if self.m_cycles.kinematicCycles is not None:
            if "Left" in self.m_kinematicLabelsDict.keys():
                for label in self.m_kinematicLabelsDict["Left"]:
                    labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                    out[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kinematicCycles,labelPlus,"Left")

                for label in CGM2cycle.GaitCycle.STP_LABELS:
                    outPst[label,"Left"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.kinematicCycles,label,"Left")

                logging.info("left kinematic computation---> done")
            else:
                logging.warning("No left Kinematic computation")

            if "Right" in self.m_kinematicLabelsDict.keys():
                for label in self.m_kinematicLabelsDict["Right"]:
                    labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                    out[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kinematicCycles,labelPlus,"Right")

                for label in CGM2cycle.GaitCycle.STP_LABELS:
                    outPst[label,"Right"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.kinematicCycles,label,"Right")

                logging.info("right kinematic computation---> done")
            else:
                logging.warning("No right Kinematic computation")

        else:
            logging.warning("No Kinematic computation")

        return out,outPst


    def computeKinetics(self):
        """ compute descriptive of kinetics parameters

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of kinetics parameters
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching  kinetics parameters

        """


        out={}
        outPst={}
        outOptional={}

        logging.info("--kinetic computation--")
        if self.m_cycles.kineticCycles is not None:

           found_context = list()
           for cycle in self.m_cycles.kineticCycles:
               found_context.append(cycle.context)


           if "Left" in self.m_kineticLabelsDict.keys():
               if "Left" in found_context:
                   for label in self.m_kineticLabelsDict["Left"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       out[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Left")
                   for label in CGM2cycle.GaitCycle.STP_LABELS:
                        outPst[label,"Left"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.kineticCycles,label,"Left")
                   for label in self.m_kinematicLabelsDict["Left"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       outOptional[labelPlus,"Left"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Left")
                   logging.info("left kinetic computation---> done")
               else:
                   logging.warning("No left Kinetic computation")




           if "Right" in self.m_kineticLabelsDict.keys():
               if  "Right" in found_context:
                   for label in self.m_kineticLabelsDict["Right"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       out[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Right")

                   for label in CGM2cycle.GaitCycle.STP_LABELS:
                        outPst[label,"Right"]=CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.kineticCycles,label,"Right")

                   for label in self.m_kinematicLabelsDict["Right"]:
                       labelPlus = label + "_" + self.m_pointlabelSuffix if self.m_pointlabelSuffix is not None else label
                       outOptional[labelPlus,"Right"]=CGM2cycle.point_descriptiveStats(self.m_cycles.kineticCycles,labelPlus,"Right")


                   logging.info("right kinetic computation---> done")
               else:
                   logging.warning("No right Kinetic computation")


        else:
            logging.warning("No Kinetic computation")

        return out,outPst,outOptional

    def computeEmgEnvelopes(self):

        """
            Compute descriptive of emg values

            :return:
                - `out` (dict) - dictionnary with descriptive statictics of emg envelopes
                - `outPst` ( dict) - dictionnary with descriptive statictics of spatio-temporal parameters matching emg envelopes
        """


        out={}
        outPst={}

        logging.info("--emg computation--")
        if self.m_cycles.emgCycles is not None:

            for rawLabel in self.m_emgLabelList:
                out[rawLabel,"Left"]=CGM2cycle.analog_descriptiveStats(self.m_cycles.emgCycles,rawLabel,"Left")
                out[rawLabel,"Right"]=CGM2cycle.analog_descriptiveStats(self.m_cycles.emgCycles,rawLabel,"Right")


            for label in CGM2cycle.GaitCycle.STP_LABELS:
                outPst[label,"Left"]= CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.emgCycles,label,"Left")
                outPst[label,"Right"]= CGM2cycle.spatioTemporelParameter_descriptiveStats(self.m_cycles.emgCycles,label,"Right")

        else:
            logging.warning("No emg computation")

        return out,outPst

# ---- FILTERS -----

class AnalysisFilter(object):
    """
         Filter building an Analysis instance.
    """

    def __init__(self):
        self.__concreteAnalysisBuilder = None
        self.analysis = Analysis()

        self.subjectInfo=None
        self.experimentalInfo=None
        self.modelInfo=None

    def setBuilder(self,concreteBuilder):
        """
             set a concrete builder

            :Parameters:
                - `concreteBuilder` (Builder) - a concrete Builder

        """

        self.__concreteAnalysisBuilder = concreteBuilder


    def setInfo(self,subject=None,experimental=None,model=None ):

        if subject is not None:
            self.subjectInfo = subject

        if experimental  is not None:
            self.experimentalInfo = experimental

        if model is not None:
            self.modelInfo = model




    def build (self) :
        """
            build member analysis from a concrete builder

        """
        pstOut = self.__concreteAnalysisBuilder.computeSpatioTemporel()
        self.analysis.setStp(pstOut)

        kinematicOut,matchPst_kinematic = self.__concreteAnalysisBuilder.computeKinematics()
        self.analysis.setKinematic(kinematicOut, pst= matchPst_kinematic)

        kineticOut,matchPst_kinetic,matchKinematic = self.__concreteAnalysisBuilder.computeKinetics()
        self.analysis.setKinetic(kineticOut, pst= matchPst_kinetic, optionalData=matchKinematic)

        if self.__concreteAnalysisBuilder.m_emgLabelList :
            emgOut,matchPst_emg = self.__concreteAnalysisBuilder.computeEmgEnvelopes()
            self.analysis.setEmg(emgOut, pst = matchPst_emg)

        if self.subjectInfo is not None:
            self.analysis.setSubjectInfo(self.subjectInfo)

        if self.experimentalInfo is not None:
            self.analysis.setExperimentalInfo(self.experimentalInfo)

        if self.modelInfo is not None:
            self.analysis.setModelInfo(self.modelInfo)
